import py
from sympy import *
from sympy.numerics import *
from sympy.numerics.optimize import *
from sympy.utilities.pytest import XFAIL

@XFAIL
def test_polyroots():
    x = Symbol('x')
    rs = polyroots(4*(x-3)*(x+I)*(x-4-5*I))
    assert rs[0][0].ae(ComplexFloat(0, -1))
    assert rs[0][1].ae(ComplexFloat(3, 0))
    assert rs[0][2].ae(ComplexFloat(4, 5))

def test_bisect():
    a, b = bisect(sin, 3, 4)
    assert sin(a).ae(0)
    assert sin(b).ae(0)
    assert sin(a)*sin(b) <= 0

def test_secant():
    assert secant(lambda x: x**2-4, 3.7).ae(2)
    Float.setdps(100)
    assert secant(sin, 3).ae(pi_float())
    Float.setdps(15)




from monomial import monomials, monomial_count
from polynomial import Poly, PolynomialError

from algorithms import poly_div, poly_pdiv

def poly_quo(f, g, *symbols):
    return poly_div(f, g, *symbols)[0]

def poly_rem(f, g, *symbols):
    return poly_div(f, g, *symbols)[1]

def poly_pquo(f, g, *symbols):
    return poly_pdiv(f, g, *symbols)[0]

def poly_prem(f, g, *symbols):
    return poly_pdiv(f, g, *symbols)[1]

from algorithms import poly_gcdex, poly_half_gcdex

from algorithms import poly_lcm, poly_gcd

from algorithms import poly_subresultants
from algorithms import poly_resultant

from algorithms import poly_groebner

from algorithms import poly_sqf

from algorithms import poly_decompose


import py
from sympy import *
from sympy.numerics import *
from sympy.numerics.functions import exp, log

def test_import():
    assert Float(3) + Float(6) == 9
    assert isinstance(exp(3), Float)
    assert isinstance(log(3), Float)



import py
from sympy import *
from sympy.numerics import *
from sympy.utilities.pytest import XFAIL

def test_bitcount():
    assert bitcount(0) == 0
    assert bitcount(1) == 1
    assert bitcount(7) == 3
    assert bitcount(8) == 4
    assert bitcount(2**100) == 101
    assert bitcount(2**100-1) == 100
    assert bitcount(-(2**100)) == 101
    assert bitcount(-(2**100-1)) == 100

def test_trailing_zeros():
    assert trailing_zeros(0) == 0
    assert trailing_zeros(1) == 0
    assert trailing_zeros(2) == 1
    assert trailing_zeros(7) == 0
    assert trailing_zeros(8) == 3
    assert trailing_zeros(2**100) == 100
    assert trailing_zeros(2**100-1) == 0

def test_round_down():
    assert normalize(0, -4, 4, ROUND_DOWN)[:2] == (0, 0)
    assert normalize(0xf0, -4, 4, ROUND_DOWN)[:2] == (15, 0)
    assert normalize(0xf1, -4, 4, ROUND_DOWN)[:2] == (15, 0)
    assert normalize(0xff, -4, 4, ROUND_DOWN)[:2] == (15, 0)
    assert normalize(-0xf0, -4, 4, ROUND_DOWN)[:2] == (-15, 0)
    assert normalize(-0xf1, -4, 4, ROUND_DOWN)[:2] == (-15, 0)
    assert normalize(-0xff, -4, 4, ROUND_DOWN)[:2] == (-15, 0)

def test_round_up():
    assert normalize(0, -4, 4, ROUND_UP)[:2] == (0, 0)
    assert normalize(0xf0, -4, 4, ROUND_UP)[:2] == (15, 0)
    assert normalize(0xf1, -4, 4, ROUND_UP)[:2] == (1, 4)
    assert normalize(0xff, -4, 4, ROUND_UP)[:2] == (1, 4)
    assert normalize(-0xf0, -4, 4, ROUND_UP)[:2] == (-15, 0)
    assert normalize(-0xf1, -4, 4, ROUND_UP)[:2] == (-1, 4)
    assert normalize(-0xff, -4, 4, ROUND_UP)[:2] == (-1, 4)

def test_round_floor():
    assert normalize(0, -4, 4, ROUND_FLOOR)[:2] == (0, 0)
    assert normalize(0xf0, -4, 4, ROUND_FLOOR)[:2] == (15, 0)
    assert normalize(0xf1, -4, 4, ROUND_FLOOR)[:2] == (15, 0)
    assert normalize(0xff, -4, 4, ROUND_FLOOR)[:2] == (15, 0)
    assert normalize(-0xf0, -4, 4, ROUND_FLOOR)[:2] == (-15, 0)
    assert normalize(-0xf1, -4, 4, ROUND_FLOOR)[:2] == (-1, 4)
    assert normalize(-0xff, -4, 4, ROUND_FLOOR)[:2] == (-1, 4)

def test_round_ceiling():
    assert normalize(0, -4, 4, ROUND_CEILING)[:2] == (0, 0)
    assert normalize(0xf0, -4, 4, ROUND_CEILING)[:2] == (15, 0)
    assert normalize(0xf1, -4, 4, ROUND_CEILING)[:2] == (1, 4)
    assert normalize(0xff, -4, 4, ROUND_CEILING)[:2] == (1, 4)
    assert normalize(-0xf0, -4, 4, ROUND_CEILING)[:2] == (-15, 0)
    assert normalize(-0xf1, -4, 4, ROUND_CEILING)[:2] == (-15, 0)
    assert normalize(-0xff, -4, 4, ROUND_CEILING)[:2] == (-15, 0)

def test_round_half_up():
    assert normalize(0, -4, 4, ROUND_HALF_UP)[:2] == (0, 0)
    assert normalize(0xf0, -4, 4, ROUND_HALF_UP)[:2] == (15, 0)
    assert normalize(0xf7, -4, 4, ROUND_HALF_UP)[:2] == (15, 0)
    assert normalize(0xf8, -4, 4, ROUND_HALF_UP)[:2] == (1, 4)
    assert normalize(0xf9, -4, 4, ROUND_HALF_UP)[:2] == (1, 4)
    assert normalize(0xff, -4, 4, ROUND_HALF_UP)[:2] == (1, 4)
    assert normalize(-0xf0, -4, 4, ROUND_HALF_UP)[:2] == (-15, 0)
    assert normalize(-0xf7, -4, 4, ROUND_HALF_UP)[:2] == (-15, 0)
    assert normalize(-0xf8, -4, 4, ROUND_HALF_UP)[:2] == (-1, 4)
    assert normalize(-0xf9, -4, 4, ROUND_HALF_UP)[:2] == (-1, 4)
    assert normalize(-0xff, -4, 4, ROUND_HALF_UP)[:2] == (-1, 4)

def test_round_half_down():
    assert normalize(0, -4, 4, ROUND_HALF_DOWN)[:2] == (0, 0)
    assert normalize(0xf0, -4, 4, ROUND_HALF_DOWN)[:2] == (15, 0)
    assert normalize(0xf7, -4, 4, ROUND_HALF_DOWN)[:2] == (15, 0)
    assert normalize(0xf8, -4, 4, ROUND_HALF_DOWN)[:2] == (15, 0)
    assert normalize(0xf9, -4, 4, ROUND_HALF_DOWN)[:2] == (1, 4)
    assert normalize(0xff, -4, 4, ROUND_HALF_DOWN)[:2] == (1, 4)
    assert normalize(-0xf0, -4, 4, ROUND_HALF_DOWN)[:2] == (-15, 0)
    assert normalize(-0xf7, -4, 4, ROUND_HALF_DOWN)[:2] == (-15, 0)
    assert normalize(-0xf8, -4, 4, ROUND_HALF_DOWN)[:2] == (-15, 0)
    assert normalize(-0xf9, -4, 4, ROUND_HALF_DOWN)[:2] == (-1, 4)
    assert normalize(-0xff, -4, 4, ROUND_HALF_DOWN)[:2] == (-1, 4)

def test_round_half_even():
    assert normalize(0, -4, 4, ROUND_HALF_EVEN)[:2] == (0, 0)
    assert normalize(0xf0, -4, 4, ROUND_HALF_EVEN)[:2] == (15, 0)
    assert normalize(0xf7, -4, 4, ROUND_HALF_EVEN)[:2] == (15, 0)
    assert normalize(0xf8, -4, 4, ROUND_HALF_EVEN)[:2] == (1, 4)    # 1111.1000 -> 10000.0
    assert normalize(0xf9, -4, 4, ROUND_HALF_EVEN)[:2] == (1, 4)    # 1111.1001 -> 10000.0
    assert normalize(0xe8, -4, 4, ROUND_HALF_EVEN)[:2] == (7, 1)    # 1110.1000 -> 1110.0
    assert normalize(0xe9, -4, 4, ROUND_HALF_EVEN)[:2] == (15, 0)     # 1110.1001 -> 1111.0
    assert normalize(-0xf0, -4, 4, ROUND_HALF_EVEN)[:2] == (-15, 0)
    assert normalize(-0xf7, -4, 4, ROUND_HALF_EVEN)[:2] == (-15, 0)
    assert normalize(-0xf8, -4, 4, ROUND_HALF_EVEN)[:2] == (-1, 4)
    assert normalize(-0xf9, -4, 4, ROUND_HALF_EVEN)[:2] == (-1, 4)
    assert normalize(-0xe8, -4, 4, ROUND_HALF_EVEN)[:2] == (-7, 1)
    assert normalize(-0xe9, -4, 4, ROUND_HALF_EVEN)[:2] == (-15, 0)

def test_cmp():
    import random
    random.seed(123)
    for i in range(100):
        x = (random.random()*1000 - 1000) * 2.0**random.randint(-20, 20)
        y = (random.random()*1000 - 1000) * 2.0**random.randint(-20, 20)
        assert (Float(x) < Float(y)) == (x < y)
        assert (Float(x) > Float(y)) == (x > y)
        assert (Float(x) == Float(y)) == (x == y)
        assert (Float(x) <= Float(y)) == (x <= y)
        assert (Float(x) >= Float(y)) == (x >= y)
        assert Float(x) == Float(x)
    assert Float(4.4408920985006262E-16) < Float(1.7763568394002505E-15)
    assert Float(-4.4408920985006262E-16) > Float(-1.7763568394002505E-15)

@XFAIL
def test_almost_equal():
    assert Float(1.2).ae(Float(1.20000001), 1e-7)
    assert not Float(1.2).ae(Float(1.20000001), 1e-9)
    assert not Float(-0.7818314824680298).ae(Float(-0.774695868667929))

def test_convert():
    for i in range(-100, 100):
        assert int(Float(i)) == i
    assert int(Float(2**500 + 23)) == 2**500
    assert Float(0.5).rational() == Rational(1,2)
    assert Float(-3).rational() == Rational(-3)

@XFAIL
def test_add():
    assert Float(4) + Float(-70) == -66
    assert Float(1) + Float(1.1)/80 == 1 + 1.1/80
    assert Float((1, 10000000000)) + Float(3) == Float((1, 10000000000))
    assert Float(3) + Float((1, 10000000000)) == Float((1, 10000000000))
    assert Float((1, -10000000000)) + Float(3) == Float(3)
    assert Float(3) + Float((1, -10000000000)) == Float(3)
    assert Float(1) + 1e-15 != 1
    assert Float(1) + 1e-20 == 1
    assert Float(1.07e-22) + 0 == Float(1.07e-22)
    assert Float(0) + Float(1.07e-22) == Float(1.07e-22)

@XFAIL
def test_contexts():
    Float.store()
    Float.setprec(100)
    Float.store()
    Float.setprec(200)
    assert Float.getprec() == 200
    Float.revert()
    assert Float.getprec() == 100
    Float.revert()
    assert Float.getprec() == 53
    Float.revert()
    assert Float.getprec() == 53

@XFAIL
def test_complex():
    # many more tests needed
    assert 1 + ComplexFloat(2) == 3
    assert not ComplexFloat(2).ae(2+1e-13)
    assert ComplexFloat(2+1e-15j).ae(2)



"""
This module implements a class Float for arbitrary-precision binary
floating-point arithmetic. It is typically 10-100 times faster
than Python's standard Decimals. For details on usage, refer to the
docstrings in the Float class. A ComplexFloat class is also provided.
"""

import math
_clog = math.log
_csqrt = math.sqrt

from sympy import Basic, Rational
from utils_ import bitcount, trailing_zeros

#----------------------------------------------------------------------
# Rounding modes
#

class _RoundingMode(int):
    def __new__(cls, level, name):
        a = int.__new__(cls, level)
        a.name = name
        return a
    def __repr__(self):
        return self.name

ROUND_DOWN    = _RoundingMode(1, 'ROUND_DOWN')
ROUND_UP      = _RoundingMode(2, 'ROUND_UP')
ROUND_FLOOR   = _RoundingMode(3, 'ROUND_FLOOR')
ROUND_CEILING = _RoundingMode(4, 'ROUND_CEILING')
ROUND_HALF_UP = _RoundingMode(5, 'ROUND_HALF_UP')
ROUND_HALF_DOWN = _RoundingMode(6, 'ROUND_HALF_DOWN')
ROUND_HALF_EVEN = _RoundingMode(7, 'ROUND_HALF_EVEN')


#----------------------------------------------------------------------
# Helper functions for bit manipulation
#

def rshift(x, n, mode):
    """
    Shift x n bits to the right (i.e., calculate x/(2**n)), and
    round to the nearest integer in accordance with the specified
    rounding mode. The exponent n may be negative, in which case x is
    shifted to the left (and no rounding is necessary).
    """
    if n == 0 or x == 0: return x
    if n < 0:  return x << -n

    # Bit-fiddling is relatively expensive in Python. To get away easily, we
    # can exploit the fact that Python rounds positive integers toward
    # zero and negative integers away from zero when dividing/shifting

    # These cases can be handled by simple shifting
    if mode < ROUND_HALF_UP:
        if mode == ROUND_DOWN:
            if x > 0: return x >> n
            else:     return -((-x) >> n)
        if mode == ROUND_UP:
            if x > 0: return -((-x) >> n)
            else:     return x >> n
        if mode == ROUND_FLOOR:
            return x >> n
        if mode == ROUND_CEILING:
            return -((-x) >> n)

    # Here we need to inspect the bits around the cutoff point
    if x > 0: t = x >> (n-1)
    else:     t = (-x) >> (n-1)
    if t & 1:
        if mode == ROUND_HALF_UP or \
           (mode == ROUND_HALF_DOWN and x & ((1<<(n-1))-1)) or \
           (mode == ROUND_HALF_EVEN and (t&2 or x & ((1<<(n-1))-1))):
            if x > 0:  return (t>>1)+1
            else:      return -((t>>1)+1)
    if x > 0: return t>>1
    else:     return -(t>>1)

def normalize(man, exp, prec, mode):
    """
    Normalize the binary floating-point number represented by
    man * 2**exp to the specified precision level, rounding
    according to the specified rounding mode if necessary.
    The mantissa is also stripped of trailing zero bits, and
    its bits are counted. The returned value is a tuple
    (man, exp, bc).
    """
    if not man:
        return 0, 0, 0
    bc = bitcount(man)
    if bc > prec:
        man = rshift(man, bc-prec, mode)
        exp += (bc - prec)
        bc = prec
    # Stripping trailing zeros permits faster equality testing
    if not man & 1:
        tr = trailing_zeros(man)
        if tr:
            man >>= tr
            exp += tr
            bc -= tr
    if not man:
        return 0, 0, 0
    return man, exp, bc

# Shortcut for constructing a Float from given mantissa and exponent, used by
# some of the arithmetic operators in the Float class
def makefloat(man, exp):
    return tuple.__new__(Float, normalize(man, exp, Float._prec, Float._mode))


#----------------------------------------------------------------------
# Other helper functions
#

def binary_to_decimal(man, exp, n):
    """Represent as a decimal string with at most n digits"""
    import decimal
    prec_ = decimal.getcontext().prec
    decimal.getcontext().prec = n
    if exp >= 0: d = decimal.Decimal(man) * (1<<exp)
    else:        d = decimal.Decimal(man) / (1<<-exp)
    a = str(d)
    decimal.getcontext().prec = prec_
    return a

_convratio = _clog(10,2) # 3.3219...


#---------------------------------------------------------------------------#
#                              Float class                                  #
#---------------------------------------------------------------------------#

class Float(tuple):
    """
    A Float is a rational number of the form

        man * 2**exp

    ("man" and "exp" are short for "mantissa" and "exponent"). Both man
    and exp are integers, possibly negative, and may be arbitrarily large.
    Essentially, a larger mantissa corresponds to a higher precision
    and a larger exponent corresponds to larger magnitude.

    A Float instance is represented by a tuple

        (man, exp, bc)

    where bc is the bitcount of the mantissa. The elements can be
    accessed as named properties:

        >>> x = Float(3)
        >>> x.man
        3
        >>> x.exp
        0
        >>> x.bc
        2

    When performing an arithmetic operation on two Floats, or creating a
    new Float from an existing numerical value, the result gets rounded
    to a fixed precision level, just like with ordinary Python floats.
    Unlike regular Python floats, however, the precision level can be
    set arbitrarily high. You can also change the rounding mode (all
    modes supported by Decimal are also supported by Float).

    The precision level and rounding mode are stored as properties of
    the Float class. (An individual Float instances does not have any
    precision or rounding data associated with it.) The precision level
    and rounding mode make up the current working context. You can
    change the working context through static methods of the Float
    class:

        Float.setprec(n)    -- set precision to n bits
        Float.extraprec(n)  -- increase precision by n bits
        Float.setdps(n)     -- set precision equivalent to n decimals
        Float.setmode(mode) -- set rounding mode

    Corresponding methods are available for inspection:

        Float.getprec()
        Float.getdps()
        Float.getmode()

    There are also two methods Float.store() and Float.revert(). If
    you call Float.store() before changing precision or mode, the
    old context can be restored with Float.revert(). (If Float.revert()
    is called one time too much, the default settings are restored.)
    You can nest multiple uses of store() and revert().

    (In the future, it will also be possible to use the 'with'
    statement to change contexts.)

    Note that precision is measured in bits. Since the ratio between
    binary and decimal precision levels is irrational, setprec and
    setdps work slightly differently. When you set the precision with
    setdps, the bit precision is set slightly higher than the exact
    corresponding precision to account for the fact that decimal
    numbers cannot generally be represented exactly in binary (the
    classical example is 0.1). The exact number given to setdps
    is however used by __str__ to determine number of digits to
    display. Likewise, when you set a bit precision, the decimal
    printing precision used for __str__ is set slightly lower.

    The following rounding modes are available:

        ROUND_DOWN       -- toward zero
        ROUND_UP         -- away from zero
        ROUND_FLOOR      -- towards -oo
        ROUND_CEILING    -- towards +oo
        ROUND_HALF_UP    -- to nearest; 0.5 to 1
        ROUND_HALF_DOWN  -- to nearest; 0.5 to 0
        ROUND_HALF_EVEN  -- to nearest; 0.5 to 0 and 1.5 to 2

    The rounding modes are available both as global constants defined
    in this module and as properties of the Float class, e.g.
    Float.ROUND_CEILING.

    The default precision level is 53 bits and the default rounding
    mode is ROUND_HALF_EVEN. In this mode, Floats should round exactly
    like regular Python floats (in the absence of bugs!).
    """

    #------------------------------------------------------------------
    # Static methods for context management
    #

    # Also make these constants available from the class
    ROUND_DOWN = ROUND_DOWN
    ROUND_UP = ROUND_UP
    ROUND_FLOOR = ROUND_FLOOR
    ROUND_CEILING = ROUND_CEILING
    ROUND_HALF_UP = ROUND_HALF_UP
    ROUND_HALF_DOWN = ROUND_HALF_DOWN
    ROUND_HALF_EVEN = ROUND_HALF_EVEN

    _prec = 53
    _dps = 15
    _mode = ROUND_HALF_EVEN
    _stack = []

    @staticmethod
    def store():
        """Store the current precision/rounding context. It can
        be restored by calling Float.revert()"""
        Float._stack.append((Float._prec, Float._dps, Float._mode))

    @staticmethod
    def revert():
        """Revert to last precision/rounding context stored with
        Float.store()"""
        if Float._stack:
            Float._prec, Float._dps, Float._mode = Float._stack.pop()
        else:
            Float._prec, Float._dps, Float._mode = 53, 15, ROUND_HALF_EVEN

    @staticmethod
    def setprec(n):
        """Set precision to n bits"""
        n = int(n)
        Float._prec = n
        Float._dps = int(round(n/_convratio)-1)

    @staticmethod
    def setdps(n):
        """Set the precision high enough to allow representing numbers
        with at least n decimal places without loss."""
        n = int(n)
        Float._prec = int(round((n+1)*_convratio))
        Float._dps = n

    @staticmethod
    def extraprec(n):
        Float.setprec(Float._prec + n)

    @staticmethod
    def setmode(mode):
        assert isinstance(mode, _RoundingMode)
        Float._mode = mode

    @staticmethod
    def getprec(): return Float._prec

    @staticmethod
    def getdps(): return Float._dps

    @staticmethod
    def getmode(): return Float._mode


    #------------------------------------------------------------------
    # Core object functionality
    #

    man = property(lambda self: self[0])
    exp = property(lambda self: self[1])
    bc = property(lambda self: self[2])

    def __new__(cls, x=0, prec=None, mode=None):
        """
        Float(x) creates a new Float instance with value x. The usual
        types are supported for x:

            >>> Float(3)
            Float('3')
            >>> Float(3.5)
            Float('3.5')
            >>> Float('3.5')
            Float('3.5')
            >>> Float(Rational(7,2))
            Float('3.5')

        You can also create a Float from a tuple specifying its
        mantissa and exponent:

            >>> Float((5, -3))
            Float('0.625')

        Use the prec and mode arguments to specify a custom precision
        level (in bits) and rounding mode. If these arguments are
        omitted, the current working precision is used instead.

            >>> Float('0.500001', prec=3, mode=ROUND_DOWN)
            Float('0.5')
            >>> Float('0.500001', prec=3, mode=ROUND_UP)
            Float('0.625')

        """
        prec = prec or cls._prec
        mode = mode or cls._mode
        if isinstance(x, tuple):
            return tuple.__new__(cls, normalize(x[0], x[1], prec, mode))
        elif isinstance(x, (int, long)):
            return tuple.__new__(cls, normalize(x, 0, prec, mode))
        elif isinstance(x, float):
            # We assume that a float mantissa has 53 bits
            m, e = math.frexp(x)
            return tuple.__new__(cls, normalize(int(m*(1<<53)), e-53, prec, mode))
        elif isinstance(x, (str, Rational)):
            if isinstance(x, str):
                x = Rational(x)
            n = prec + bitcount(x.q) + 2
            return tuple.__new__(cls, normalize((x.p<<n)//x.q, -n, prec, mode))
        else:
            raise TypeError

    def __hash__(s):
        try:
            # Try to be compatible with hash values for floats and ints
            return hash(float(s))
        except OverflowError:
            # We must unfortunately sacrifice compatibility with ints here. We
            # could do hash(man << exp) when the exponent is positive, but
            # this would cause unreasonable inefficiency for large numbers.
            return hash(self.man) + hash(self.exp)

    def __pos__(s):
        """s.__pos__() <==> +s

        Normalize s to the current working precision, rounding according
        to the current rounding mode."""
        return Float(s)

    def __float__(s):
        """Convert s to a Python float. OverflowError will be raised
        if the magnitude of s is too large."""
        try:
            return math.ldexp(s.man, s.exp)
        # Handle case when mantissa has too many bits (will still
        # overflow if exp is too large)
        except OverflowError:
            n = s.bc - 64
            m = s.man >> n
            return math.ldexp(m, s.exp + n)

    def __int__(s):
        """Convert to a Python int, using floor rounding"""
        return rshift(s.man, -s.exp, 0)

    def rational(s):
        """Convert to a SymPy Rational"""
        if s.exp > 0:
            return Rational(s.man * 2**s.exp, 1)
        else:
            return Rational(s.man, 2**(-s.exp))

    def __repr__(s):
        """Represent s as a decimal string, with sufficiently many
        digits included to ensure that Float(repr(s)) == s at the
        current working precision."""
        st = "Float('%s')"
        return st % binary_to_decimal(s.man, s.exp, Float._dps + 2)

    def __str__(s):
        """Print slightly more prettily than __repr__"""
        return binary_to_decimal(s.man, s.exp, Float._dps)


    #------------------------------------------------------------------
    # Comparisons
    #

    def __cmp__(s, t):
        """s.__cmp__(t) <==> cmp(s, t)

        If t is a Float, int or float, this returns -1 if s < t, 0 if
        s == t, and 1 if s > t. The comparison operators >, >=, <, <=
        are defined in terms of this function. If t is a SymPy Basic
        instance, s is converted into a Rational and Rational.__cmp__
        is called.

        Warning: in extreme cases, the truncation error resulting from
        calling Float(t) will result in an erroneous comparison: for
        example, Float(2**80) will compare as equal to 2**80+1. This
        problem can be circumvented by manually increasing the working
        precision or by converting numbers into Rationals for
        comparisons.
        """

        if not isinstance(t, Float):
            if isinstance(t, Basic):
                return s.rational().__cmp__(t)
            t = Float(t)

        # Note: the reason we call cmp(x,y) below instead of directly forming
        # the integer x-y is that the output from __cmp__ must be a machine
        # size integer in Python.

        # An inequality between two numbers s and t is determined by looking
        # at the value of s-t. A full floating-point subtraction is relatively
        # slow, so we first try to look at the exponents and signs of s and t.
        sm, se, sbc = s
        tm, te, tbc = t

        # Very easy cases: check for 0's and opposite signs
        if not tm: return cmp(sm, 0)
        if not sm: return cmp(0, tm)
        if sm > 0 and tm < 0: return 1
        if sm < 0 and tm > 0: return -1

        # In this case, the numbers likely have the same magnitude
        if se == te: return cmp(sm, tm)

        # The numbers have the same sign but different exponents. In this
        # case we try to determine if they are of different magnitude by
        # checking the position of the highest set bit in each number.
        a = sbc + se
        b = tbc + te
        if sm > 0:
            if a < b: return -1
            if a > b: return 1
        else:
            if a < b: return 1
            if a < b: return -1

        # The numbers have similar magnitude but different exponents.
        # So we subtract and check the sign of resulting mantissa.
        return cmp((s-t)[0], 0)


    # Since Float inherits from tuple, these functions must be defined
    # in addition to __cmp__ to override the tuple methods.
    __lt__ = lambda s, t: s.__cmp__(t) < 0
    __gt__ = lambda s, t: s.__cmp__(t) > 0
    __le__ = lambda s, t: s.__cmp__(t) <= 0
    __ge__ = lambda s, t: s.__cmp__(t) >= 0


    """
    We implement __eq__ separately from __cmp__. One reason for doing this
    performance: due to the way Floats are normalized, two Floats are
    mathematically equal iff all three parts (man, exp, bc) are equal. So we
    can do a direct tuple comparison and avoid the machinery in __cmp__.

    Another reason is to support == and != between Floats and ComplexFloat.
    ComplexFloats are not supported by __cmp__, since complex numbers are
    unordered.
    """
    def __eq__(s, t):
        """s.__eq__(t) <==> s == Float(t)

        Determine whether s and Float(t) are equal (see warning for
        __cmp__ about conversion between different types.)"""
        if isinstance(t, Basic):
            return s.rational() == t
        if isinstance(t, Float):
            return s[:] == t[:]
        if isinstance(t, ComplexFloat):
            return ComplexFloat(s) == t
        return not s.__cmp__(t)

    __ne__ = lambda s, t: not s.__eq__(t)

    def ae(s, t, rel_eps=None, abs_eps=None):
        """
        Determine whether the difference between s and t is smaller
        than a given epsilon ("ae" is short for "almost equal").

        Both a maximum relative difference and a maximum difference
        ('epsilons') may be specified. The absolute difference is
        defined as |s-t| and the relative difference is defined
        as |s-t|/max(|s|, |t|).

        If only one epsilon is given, both are set to the same value.
        If none is given, both epsilons are set to 2**(-prec+m) where
        prec is the current working precision and m is a small integer.
        """
        if not isinstance(t, Float):
            t = Float(t)
        if abs_eps is None and rel_eps is None:
            rel_eps = tuple.__new__(Float, (1, -s._prec+4, 1))
            rel_eps = Float((1, -s._prec+4))
        if abs_eps is None:
            abs_eps = rel_eps
        elif rel_eps is None:
            rel_eps = abs_eps
        diff = abs(s-t)
        if diff <= abs_eps:
            return True
        abss = abs(s)
        abst = abs(t)
        if abss < abst:
            err = diff/abst
        else:
            err = diff/abss
        return err <= rel_eps

    def almost_zero(s, prec):
        """Quick check if |s| < 2**-prec. May return a false negative
        if s is very close to the threshold."""
        return s.bc + s.exp < prec

    def __nonzero__(s):
        return bool(s[0])

    #------------------------------------------------------------------
    # Arithmetic
    #

    def __abs__(s):
        if s[0] < 0:
            return -s
        return s

    def __add__(s, t):
        if isinstance(t, Float):
            if t[1] > s[1]:
                s, t = t, s
            sman, sexp, sbc = s
            tman, texp, tbc = t
            if not tman: return +s
            if not sman: return +t
            if sexp - texp > 100:
                bitdelta = (sbc+sexp)-(tbc+texp)
                if bitdelta > s._prec+5:
                    # XXX: handle rounding
                    return +s
            return makefloat(tman+(sman<<(sexp-texp)), texp)
        if isinstance(t, (int, long)):
            # XXX: cancellation is possible here
            return s + Float(t)
        if isinstance(t, (ComplexFloat, complex)):
            return ComplexFloat(s) + t
        return s + Float(t)

    __radd__ = __add__

    def __neg__(s):
        return makefloat(-s[0], s[1])

    def __sub__(s, t):
        if isinstance(t, Float):
            return s + tuple.__new__(Float, (-t[0],) + t[1:])
        else:
            return s + (-t)

    def __rsub__(s, t):
        return (-s) + t

    def __mul__(s, t):
        if isinstance(t, Float):
            sman, sexp, sbc = s
            tman, texp, tbc = t
            return makefloat(sman*tman, sexp+texp)
        if isinstance(t, (int, long)):
            return makefloat(s[0]*t, s[1])
        if isinstance(t, (ComplexFloat, complex)):
            return ComplexFloat(s) * t
        return s * Float(t)

    __rmul__ = __mul__

    def __div__(s, t):
        if t == 0:
            raise ZeroDivisionError
        if isinstance(t, Float):
            sman, sexp, sbc = s
            tman, texp, tbc = t
            extra = max(0, s._prec - sbc + tbc + 4)
            return makefloat((sman<<extra)//tman, sexp-texp-extra)
        if isinstance(t, (int, long)):
            sman, sexp, sbc = s
            extra = s._prec - sbc + bitcount(t) + 4
            return makefloat((sman<<extra)//t, sexp-extra)
        if isinstance(t, (ComplexFloat, complex)):
            return ComplexFloat(s) / t
        return s / Float(t)

    def __rdiv__(s, t):
        if isinstance(t, (ComplexFloat, complex)):
            return t / ComplexFloat(s)
        return Float(t) / s

    def __pow__(s, n):
        """Calculate (man*2**exp)**n, currently for integral n only."""
        if isinstance(n, (int, long)):
            if n == 0: return Float((1, 0))
            if n == 1: return +s
            if n == 2: return s * s
            if n == -1: return Float(1) / s
            if n < 0:
                Float._prec += 2
                r = Float(1) / (s ** -n)
                Float._prec -= 2
                return +r
            else:
                prec2 = Float._prec + int(4*_clog(n, 2) + 4)
                man, exp, bc = normalize(s.man, s.exp, prec2, ROUND_FLOOR)
                pm, pe, bc = 1, 0, 1
                while n:
                    if n & 1:
                        pm, pe, bc = normalize(pm*man, pe+exp, prec2, ROUND_FLOOR)
                        n -= 1
                    man, exp, _ = normalize(man*man, exp+exp, prec2, ROUND_FLOOR)
                    n = n // 2
                #return Float((pm, pe))
                return makefloat(pm, pe)
        # TODO: support arbitrary powers through exp
        if n == 0.5:
            from functions import sqrt
            return sqrt(s)
        raise ValueError


class ComplexFloat(object):

    def __init__(s, real=0, imag=0):
        if isinstance(real, (complex, ComplexFloat)):
            real, imag = real.real, real.imag
        s.real = Float(real)
        s.imag = Float(imag)

    def __repr__(s):
        r = repr(s.real)[6:-1]
        i = repr(s.imag)[6:-1]
        return "ComplexFloat(real=%s, imag=%s)" % (r, i)

    def __str__(s):
        return "(%s + %s*I)" % (s.real, s.imag)

    def __complex__(s):
        return complex(float(s.real), float(s.imag))

    def __pos__(s):
        return ComplexFloat(s.real, s.imag)

    def __abs__(s):
        from functions import hypot
        return hypot(s.real, s.imag)

    def __eq__(s, t):
        if not isinstance(t, ComplexFloat):
            t = ComplexFloat(t)
        return s.real == t.real and s.imag == t.imag

    # TODO: refactor and merge with Float.ae
    def ae(s, t, rel_eps=None, abs_eps=None):
        if not isinstance(t, ComplexFloat):
            t = ComplexFloat(t)
        if abs_eps is None and rel_eps is None:
            rel_eps = Float((1, -s.real._prec+4))
        if abs_eps is None:
            abs_eps = rel_eps
        elif rel_eps is None:
            rel_eps = abs_eps
        diff = abs(s-t)
        if diff <= abs_eps:
            return True
        abss = abs(s)
        abst = abs(t)
        if abss < abst:
            err = diff/abst
        else:
            err = diff/abss
        return err <= rel_eps

    def __nonzero__(s):
        return s.real != 0 or s.imag != 0

    def conjugate(s):
        return ComplexFloat(s.real, -s.imag)

    def __add__(s, t):
        if not isinstance(t, ComplexFloat):
            t = ComplexFloat(t)
        return ComplexFloat(s.real+t.real, s.imag+t.imag)

    __radd__ = __add__

    def __neg__(s):
        return ComplexFloat(-s.real, -s.imag)

    def __sub__(s, t):
        if not isinstance(t, ComplexFloat):
            t = ComplexFloat(t)
        return ComplexFloat(s.real-t.real, s.imag-t.imag)

    def __rsub__(s, t):
        return (-s) + t

    def __mul__(s, t):
        if not isinstance(t, ComplexFloat):
            t = ComplexFloat(t)
        a = s.real; b = s.imag; c = t.real; d = t.imag
        if b == d == 0:
            return ComplexFloat(a*c, 0)
        else:
            return ComplexFloat(a*c-b*d, a*d+b*c)

    __rmul__ = __mul__

    def __div__(s, t):
        if not isinstance(t, ComplexFloat):
            t = ComplexFloat(t)
        a = s.real; b = s.imag; c = t.real; d = t.imag
        mag = c*c + d*d
        return ComplexFloat((a*c+b*d)/mag, (b*c-a*d)/mag)

    def __rdiv__(s, t):
        return ComplexFloat(t) / s

    def __pow__(s, n):
        if n == 0: return ComplexFloat(1)
        if n == 1: return +s
        if n == -1: return 1/s
        if n == 2: return s*s
        if isinstance(n, (int, long)) and n > 0:
            # TODO: should increase working precision here
            w = ComplexFloat(1)
            while n:
                if n & 1:
                    w = w*s
                    n -= 1
                s = s*s
                n //= 2
            return w
        raise NotImplementedError



from math import log as _clog

_bc_table = (0, 1, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4)

def bitcount(n, log=_clog, bctable=_bc_table):
    """Give position of the highest set bit in an integer"""
    if not n: return 0
    if n < 0: n = -n
    # math.log gives a good estimate, and never overflows, but
    # is not always exact. Subtract 2 to underestimate, then
    # count remaining bits by table lookup
    bc = int(log(n, 2)) - 2
    if bc < 0:
        bc = 0
    return bc + bctable[n >> bc]

def trailing_zeros(n):
    """Count trailing zero bits in an integer."""
    if n & 1: return 0
    if not n: return 0
    if n < 0: n = -n
    t = 0
    while not n & 0xffffffffffffffff: n >>= 64; t += 64
    while not n & 0xff: n >>= 8; t += 8
    while not n & 1: n >>= 1; t += 1
    return t

def make_fixed(x, prec):
    """Convert a Float to a fixed-point big integer"""
    offset = x.exp+prec
    if offset >= 0:
        return x.man << offset
    else:
        return x.man >> (-offset)

def bin_to_radix(x, xbits, base, bdigits):
    return x * (base**bdigits) >> xbits

_numerals = '0123456789abcdefghijklmnopqrstuvwxyz'

def small_numeral(n, base=10):
    # Calculate numeral of n*(base**digits) in the given base
    if base == 10:
        return str(n)
    digs = []
    while n:
        n, digit = divmod(n, base)
        digs.append(_numerals[digit])
    return "".join(digs[::-1])

# TODO: speed up for bases 2, 4, 8, 16, ...
def fixed_to_str(x, base, digits):
    if digits < 789:
        return small_numeral(x, base)
    half = (digits // 2) + (digits & 1)
    if "verbose" in global_options and half > 50000:
        print "  dividing..."
    A, B = divmod(x, base**half)
    ad = fixed_to_str(A, base, half)
    bd = fixed_to_str(B, base, half).rjust(half, "0")
    return ad + bd

global_options = {}



"""
Numerical implementations of special functions (gamma, ...)
"""

from float_ import Float, ComplexFloat
from constants import pi_float, gamma_float
from functions import exp, log, sqrt, sin, power
from utils_ import make_fixed

from sympy import Rational


#---------------------------------------------------------------------------#
#                                                                           #
#                              Gamma function                               #
#                                                                           #
#---------------------------------------------------------------------------#

"""
We compute the gamma function using Spouge's approximation

    x! = (x+a)**(x+1/2) * exp(-x-a) * [c_0 + S(x) + eps]

where S(x) is the sum of c_k/(x+k) from k = 1 to a-1 and the coefficients
are given by

  c_0 = sqrt(2*pi)

         (-1)**(k-1)
  c_k =  ----------- (a-k)**(k-1/2) exp(-k+a),  k = 1,2,...,a-1
          (k - 1)!

Due to an inequality proved by Spouge, if we choose a = int(1.26*n), the
error eps is less than 10**-n for any x in the right complex half-plane
(assuming a > 2). In practice, it seems that a can be chosen quite a bit
lower still (30-50%); this possibility should be investigated.

Reference:
John L. Spouge, "Computation of the gamma, digamma, and trigamma
functions", SIAM Journal on Numerical Analysis 31 (1994), no. 3, 931-944.
"""


#----------------------------------------------------------------------
#
# We first implement a helper function for calculating the coefficients
# c_k and caching them so that they can be re-used for multiple gamma
# function evaluations
#

_spouge_cache = {}

def _calc_spouge_coefficients(a, prec):
    """
    Calculate Spouge coefficients for approximation with parameter a.
    Return a list of big integers representing the coefficients in
    fixed-point form with a precision of prec bits.
    """

    # We'll store the coefficients as fixed-point numbers but calculate
    # them as Floats for convenience. The initial terms are huge, so we
    # need to allocate extra bits to ensure full accuracy. The integer
    # part of the largest term has size ~= exp(a) or 2**(1.4*a)
    floatprec = prec + int(a*1.4)
    Float.store()
    Float.setprec(floatprec)

    c = [0] * a
    b = exp(a-1)
    e = exp(1)
    c[0] = make_fixed(sqrt(2*pi_float()), prec)
    for k in range(1, a):
        # print "%02f" % (100.0 * k / a), "% done"
        c[k] = make_fixed(((-1)**(k-1) * (a-k)**k) * b / sqrt(a-k), prec)
        # Divide off e and k instead of computing exp and k! from scratch
        b = b / (e * k)

    Float.revert()
    return c

# Cached lookup of coefficients
def _get_spouge_coefficients(prec):

    # This exact precision has been used before
    if prec in _spouge_cache:
        return _spouge_cache[prec]

    for p in _spouge_cache:
        # Coefficients calculated for a slightly higher precision are ok
        # too. But if the difference is too big, we're better off
        # starting from scratch
        if 0.8 <= float(p)/prec < 1:
            return _spouge_cache[p]

    # Here we estimate the value of a based on Spouge's inequality for
    # the relative error
    a = max(3, int(0.39*prec))  # ~= 1.26*n

    # Compute and return
    coefs = _calc_spouge_coefficients(a, prec)
    _spouge_cache[prec] = (prec, a, coefs)
    return _spouge_cache[prec]


# This function computes S
def _spouge_sum(x, prec, a, c):
    if isinstance(x, Float):
        # Regular fixed-point summation
        x = make_fixed(x, prec)
        s = c[0]
        for k in xrange(1, a):
            s += (c[k] << prec) // (x + (k << prec))
        return Float((s, -prec))
    elif isinstance(x, (Rational, int, long)):
        # Here we can save some work
        if isinstance(x, (int, long)):
            p, q = x, 1
        else:
            p, q = x.p, x.q
        s = c[0]
        for k in xrange(1, a):
            s += c[k] * q // (p+q*k)
        return Float((s, -prec))
    elif isinstance(x, ComplexFloat):
        """
        For a complex number a + b*I, we have

              c_k          (a+k)*c_k     b * c_k
        -------------  =   ---------  -  ------- * I
        (a + b*I) + k          M            M

                       2    2      2   2              2
        where M = (a+k)  + b   = (a + b ) + (2*a*k + k )
        """
        re = make_fixed(x.real, prec)
        im = make_fixed(x.imag, prec)
        sre, sim = c[0], 0
        mag = ((re**2)>>prec) + ((im**2)>>prec)
        for k in xrange(1, a):
            M = mag + re*(2*k) + ((k**2) << prec)
            sre += (c[k] * (re + (k << prec))) // M
            sim -= (c[k] * im) // M
        return ComplexFloat(Float((sre, -prec)), Float((sim, -prec)))


def gamma(x):
    """
    gamma(x) -- calculate the gamma function of a real or complex
    number x.

    x must not be a negative integer or 0
    """
    Float.store()
    Float._prec += 2

    if isinstance(x, complex):
        x = ComplexFloat(x)
    elif not isinstance(x, (Float, ComplexFloat, Rational, int, long)):
        x = Float(x)

    if isinstance(x, (ComplexFloat, complex)):
        re, im = x.real, x.imag
    else:
        re, im = x, 0

    # For negative x (or positive x close to the pole at x = 0),
    # we use the reflection formula
    if re < 0.25:
        if re == int(re) and im == 0:
            raise ZeroDivisionError, "gamma function pole"
        Float._prec += 3
        p = pi_float()
        g = p / (sin(p*x) * gamma(1-x))
    else:
        x -= 1
        prec, a, c = _get_spouge_coefficients(Float.getprec()+7)
        s = _spouge_sum(x, prec, a, c)
        if not isinstance(x, (Float, ComplexFloat)):
            x = Float(x)
        # TODO: higher precision may be needed here when the precision
        # and/or size of x are extremely large
        Float._prec += 10
        g = exp(log(x+a)*(x+Float(0.5))) * exp(-x-a) * s

    Float.revert()
    return +g


#---------------------------------------------------------------------------#
#                                                                           #
#                       Incomplete gamma functions                          #
#                                                                           #
#---------------------------------------------------------------------------#

"""
We compute the lower incomplete gamma function g(a,z) using the formula
g(a,z) = z**a * exp(-z) * S(a,z) / a, where
                 oo
                ___            k
               \              z
  S(a,z) = 1 +  )     ------------------.
               /___   (a+1)(a+2)...(a+k)
               k = 1

Then, in turn, various functions such as erf and exponential integrals
can be computed from the incomplete gamma function.
"""

def _lower_gamma_series(are, aim, zre, zim, prec):
    are = make_fixed(are, prec)
    aim = make_fixed(aim, prec)
    zre = make_fixed(zre, prec)
    zim = make_fixed(zim, prec)
    one = 1 << prec
    cre = sre = one
    cim = sim = 0
    while abs(cre) > 3 or abs(cim) > 3:
        # c = (c * z) << prec
        cre, cim = (cre*zre-cim*zim)>>prec, (cim*zre+cre*zim)>>prec
        # c = c / (a+k)
        are += one
        mag = ((are**2 + aim**2) >> prec)
        cre, cim = (cre*are + cim*aim)//mag, (cim*are - cre*aim)//mag
        sre += cre
        sim += cim
        #k += 1
    sre = Float((sre, -prec))
    sim = Float((sim, -prec))
    return ComplexFloat(sre, sim)

def lower_gamma(a, z):
    Float.store()
    prec = Float._prec
    # XXX: may need more precision
    Float._prec += 15
    a = ComplexFloat(a)
    z = ComplexFloat(z)
    s = _lower_gamma_series(a.real, a.imag, z.real, z.imag, prec)
    y = exp(log(z)*a) * exp(-z) * s / a
    Float.revert()
    return +y

def upper_gamma(a, z):
    return gamma(a) - lower_gamma(a, z)

def erf(x):
    x = ComplexFloat(x)
    if x == 0: return Float(0)
    if x.real < 0: return -erf(-x)
    Float.store()
    Float._prec += 10
    y = lower_gamma(0.5, x**2) / sqrt(pi_float())
    if x.imag == 0:
        y = y.real
    Float.revert()
    return +y


#---------------------------------------------------------------------------#
#                                                                           #
#                         Riemann zeta function                             #
#                                                                           #
#---------------------------------------------------------------------------#

"""
We use zeta(s) = eta(s) * (1 - 2**(1-s)) and Borwein's approximation
                  n-1
                  ___       k
             -1  \      (-1)  (d_k - d_n)
  eta(s) ~= ----  )     ------------------
             d_n /___              s
                 k = 0      (k + 1)
where
             k
             ___                i
            \     (n + i - 1)! 4
  d_k  =  n  )    ---------------.
            /___   (n - i)! (2i)!
            i = 0

If s = a + b*I, the absolute error for eta(s) is bounded by

    3 (1 + 2|b|)
    ------------ * exp(|b| pi/2)
               n
    (3+sqrt(8))

Disregarding the linear term, we have approximately,

  log(err) ~= log(exp(1.58*|b|)) - log(5.8**n)
  log(err) ~= 1.58*|b| - log(5.8)*n
  log(err) ~= 1.58*|b| - 1.76*n
  log2(err) ~= 2.28*|b| - 2.54*n

So for p bits, we should choose n > (p + 2.28*|b|) / 2.54.

Reference:
Peter Borwein, "An Efficient Algorithm for the Riemann Zeta Function"
http://www.cecm.sfu.ca/personal/pborwein/PAPERS/P117.ps

http://en.wikipedia.org/wiki/Dirichlet_eta_function
"""

_d_cache = {}

def _zeta_coefs(n):
    if n in _d_cache:
        return _d_cache[n]
    ds = [0] * (n+1)
    d = 1
    s = ds[0] = 1
    for i in range(1, n+1):
        d = d * 4 * (n+i-1) * (n-i+1)
        d //= ((2*i) * ((2*i)-1))
        s += d
        ds[i] = s
    _d_cache[n] = ds
    return ds

# Integer logarithms
_log_cache = {}

def _logk(k):
    p = Float._prec
    if k in _log_cache and _log_cache[k][0] >= p:
        return +_log_cache[k][1]
    else:
        x = log(k)
        _log_cache[k] = (p, x)
        return x

def zeta(s):
    """
    zeta(s) -- calculate the Riemann zeta function of a real or complex
    argument s.

    """
    Float.store()
    Float._prec += 8
    si = s
    s = ComplexFloat(s)
    if s.real < 0:
        # Reflection formula (XXX: gets bad around the zeros)
        pi = pi_float()
        y = power(2, s) * power(pi, s-1) * sin(pi*s/2) * gamma(1-s) * zeta(1-s)
    else:
        p = Float._prec
        n = int((p + 2.28*abs(float(s.imag)))/2.54) + 3
        d = _zeta_coefs(n)
        if isinstance(si, (int, long)):
            t = 0
            for k in range(n):
                t += (((-1)**k * (d[k] - d[n])) << p) // (k+1)**si
            y = (Float((t, -p)) / -d[n]) / (Float(1) - Float(2)**(1-si))
        else:
            t = Float(0)
            for k in range(n):
                t += (-1)**k * Float(d[k]-d[n]) * exp(-_logk(k+1)*s)
            y = (t / -d[n]) / (Float(1) - exp(log(2)*(1-s)))
    Float.revert()
    if isinstance(y, ComplexFloat) and s.imag == 0:
        return +y.real
    else:
        return +y



from sympy import Basic, S, Symbol, Real, Integer, Rational,  \
    sin, cos, exp, log, oo, sqrt, symbols, Integral, sympify, \
    WildFunction

import py

x = Symbol("x")
y = Symbol("y")

# basic sympy objects
basic_objs = [
    Rational(2),
    Real("1.3"),
    x,
    y,
    pow(x,y)*y,
]

# all supported objects
all_objs = basic_objs + [
    5,
    5.5,
]

def dotest(s):
    for x in all_objs:
        for y in all_objs:
            s(x,y)

def test_basic():
    def s(a,b):
        x = a
        x = +a
        x = -a
        x = a+b
        x = a-b
        x = a*b
        x = a/b
        x = a**b
    dotest(s)

def test_ibasic():
    def s(a,b):
        x = a
        x += b
        x = a
        x -= b
        x = a
        x *= b
        x = a
        x /= b
    dotest(s)

def test_basic_nostr():
    for obj in basic_objs:
        for op in ['+','-','*','/','**']:
            py.test.raises(TypeError, "obj %s '1'" % op)

def test_ldegree():
    assert (1/x**2+1+x+x**2).ldegree(x)==-2
    assert (1/x+1+x+x**2).ldegree(x)==-1
    assert (x**2+1/x).ldegree(x)==-1
    assert (1+x**2).ldegree(x)==0
    assert (x+1).ldegree(x)==0
    assert (x+x**2).ldegree(x)==1
    assert (x**2).ldegree(x)==2

def test_leadterm():
    assert (3+2*x**(log(3)/log(2)-1)).leadterm(x)==(3,0)

def test_atoms():
   assert list((1+x).atoms()) == [1,x]
   assert list(x.atoms()) == [x]
   assert list((1+2*cos(x)).atoms(type=Symbol)) == [x]
   assert list((2*(x**(y**x))).atoms()) == [2,x,y]
   assert list(Rational(1,2).atoms()) == [Rational(1,2)]
   assert list(Rational(1,2).atoms(type=type(oo))) == []

def test_is_polynomial():
    z = Symbol('z')

    k = Symbol('k', nonnegative=True, integer=True)

    assert Rational(2).is_polynomial(x, y, z) == True
    assert (S.Pi).is_polynomial(x, y, z) == True

    assert x.is_polynomial(x) == True
    assert x.is_polynomial(y) == True

    assert (x**2).is_polynomial(x) == True
    assert (x**2).is_polynomial(y) == True

    assert (x**(-2)).is_polynomial(x) == False
    assert (x**(-2)).is_polynomial(y) == True

    assert (2**x).is_polynomial(x) == False
    assert (2**x).is_polynomial(y) == True

    assert (x**k).is_polynomial(x) == True
    assert (x**k).is_polynomial(k) == False
    assert (x**x).is_polynomial(x) == False
    assert (k**k).is_polynomial(k) == False
    assert (k**x).is_polynomial(k) == None

    assert (x**(-k)).is_polynomial(x) == None
    assert ((2*x)**k).is_polynomial(x) == True

    assert (x**2 + 3*x - 8).is_polynomial(x) == True
    assert (x**2 + 3*x - 8).is_polynomial(y) == True

    assert (x**2 + 3*x - 8).is_polynomial() == True

    assert sqrt(x).is_polynomial(x) == False
    assert (x**S.Half).is_polynomial(x) == False
    assert (x**Rational(3,2)).is_polynomial(x) == False

    assert (x**2 + 3*x*sqrt(y) - 8).is_polynomial(x) == True
    assert (x**2 + 3*x*sqrt(y) - 8).is_polynomial(y) == False

    assert ((x**2)*(y**2) + x*(y**2) + y*x + exp(2)).is_polynomial() == True
    assert ((x**2)*(y**2) + x*(y**2) + y*x + exp(x)).is_polynomial() == False

    assert ((x**2)*(y**2) + x*(y**2) + y*x + exp(2)).is_polynomial(x, y) == True
    assert ((x**2)*(y**2) + x*(y**2) + y*x + exp(x)).is_polynomial(x, y) == False

def test_is_fraction():
    x,y = symbols('xy')

    assert Integer(1).is_fraction() == True
    assert Integer(1).is_fraction(x) == True

    assert Rational(17,54).is_fraction() == True
    assert Rational(17,54).is_fraction(x) == True

    assert (12/x).is_fraction() == True
    assert (12/x).is_fraction(x) == True

    assert (x/y).is_fraction() == True
    assert (x/y).is_fraction(x) == True
    assert (x/y).is_fraction(x, y) == True

    assert (x**2+1/x/y).is_fraction() == True
    assert (x**2+1/x/y).is_fraction(x) == True
    assert (x**2+1/x/y).is_fraction(x, y) == True

    assert (sin(y)/x).is_fraction() == False
    assert (sin(y)/x).is_fraction(y) == False
    assert (sin(y)/x).is_fraction(x) == True
    assert (sin(y)/x).is_fraction(x, y) == False

def test_SAGE1():
    #see http://code.google.com/p/sympy/issues/detail?id=247
    class MyInt:
        def _sympy_(self):
            return Integer(5)
    m = MyInt()
    e = Rational(2)*m
    assert e == 10

    py.test.raises(TypeError, "Rational(2)*MyInt")

def test_SAGE2():
    class MyInt(object):
        def __int__(self):
            return 5
    assert sympify(MyInt()) == 5
    e = Rational(2)*MyInt()
    assert e == 10

    py.test.raises(TypeError, "Rational(2)*MyInt")

def test_SAGE3():
    class MySymbol:
        def __rmul__(self, other):
            return ('mys', other, self)

    o = MySymbol()
    e = x*o

    assert e == ('mys', x, o)

def test_ordering():
    from sympy.core.methods import ArithMeths
    class CustomClass1(Basic, ArithMeths): pass
    class CustomClass2(Basic, ArithMeths): pass
    cc1 = CustomClass1(commutative=True)
    cc2 = CustomClass2(commutative=True)

    assert str(Rational(2)*cc1) == '2*CustomClass1()'
    assert str(cc1*Rational(2)) == '2*CustomClass1()'
    assert str(cc1*Real("1.5")) == '1.5*CustomClass1()'
    assert str(cc2*Rational(2)) == '2*CustomClass2()'
    assert str(cc2*Rational(2)*cc1) == '2*CustomClass1()*CustomClass2()'
    assert str(cc1*Rational(2)*cc2) == '2*CustomClass1()*CustomClass2()'

def test_len():
    x, y, z = symbols("xyz")
    e = x*y
    assert len(e.args) == 2
    e = x+y+z
    assert len(e.args) == 3

def test_doit():
    a = Integral(x**2, x)

    assert isinstance(a.doit(), Integral) == False

    assert isinstance(a.doit(integrals=True), Integral) == False
    assert isinstance(a.doit(integrals=False), Integral) == True

    assert (2*Integral(x, x)).doit() == x**2

def test_is_number():
    assert Rational(8).is_number
    assert not x.is_number
    assert (8+log(2)).is_number
    assert not (8+log(2)+x).is_number
    assert (1+x**2/x-x).is_number

def test_attribute_error():
    py.test.raises(AttributeError, "x.cos()")
    py.test.raises(AttributeError, "x.sin()")
    py.test.raises(AttributeError, "x.exp()")

def test_args():
    assert (x*y).args[:] in ((x, y), (y, x))
    assert (x+y).args[:] in ((x, y), (y, x))
    assert (x*y+1).args[:] in ((x*y, 1), (1, x*y))
    assert sin(x*y).args[:] == (x*y,)
    assert sin(x*y).args[0] == x*y
    assert (x**y).args[:] == (x,y)
    assert (x**y).args[0] == x
    assert (x**y).args[1] == y

def test_noncommutative_expand_issue658():
    A, B, C = symbols('ABC', commutative=False)
    assert A*B - B*A != 0
    assert (A*(A+B)*B).expand() == A**2*B + A*B**2
    assert (A*(A+B+C)*B).expand() == A**2*B + A*B**2 + A*C*B

def test_as_independent():
    assert (2*x*sin(x)+y+x).as_independent(x) == (y, x + 2*x*sin(x))
    assert (2*x*sin(x)+y+x).as_independent(y) == (x + 2*x*sin(x), y)

    assert (2*x*sin(x)+y+x).as_independent(x, y) == (0, y + x + 2*x*sin(x))

    assert (x*sin(x)*cos(y)).as_independent(x) == (cos(y), x*sin(x))
    assert (x*sin(x)*cos(y)).as_independent(y) == (x*sin(x), cos(y))

    assert (x*sin(x)*cos(y)).as_independent(x, y) == (1, x*sin(x)*cos(y))

    assert (sin(x)).as_independent(x) == (1, sin(x))
    assert (sin(x)).as_independent(y) == (sin(x), 1)

    assert (2*sin(x)).as_independent(x) == (2, sin(x))
    assert (2*sin(x)).as_independent(y) == (2*sin(x), 1)

def test_subs_dict():
    a,b,c,d,e = symbols('abcde')

    assert (sin(x))._subs_dict({ x : 1, sin(x) : 2}) == 2
    assert (sin(x))._subs_dict([(x, 1), (sin(x), 2)]) == 2

    expr = sqrt(sin(2*x))*sin(exp(x)*x)*cos(2*x) + sin(2*x)

    seq = [ (sqrt(sin(2*x)),a), (cos(2*x),b), (sin(2*x),c), (x,d), (exp(x),e) ]
    assert expr._subs_dict(seq) == c + a*b*sin(d*e)

    seq = [ (sqrt(sin(2*x)),a), (sin(2*x),c), (cos(2*x),b), (x,d), (exp(x),e) ]
    assert expr._subs_dict(seq) == c + a*b*sin(d*e)

def test_subs_list():
    x,y = symbols('xy')

    assert (sin(x))._subs_list([(sin(x), 2), (x, 1)]) == 2
    assert (sin(x))._subs_list([(x, 1), (sin(x), 2)]) == sin(1)

    assert (x+y)._subs_list([(x, 3), (y, x**2)]) == 3 + x**2
    assert (x+y)._subs_list([(y, x**2), (x, 3)]) == 12


def test_has_any_symbols():
    x,y,z,t,u = symbols('xyztu')

    i = Integer(4400)

    assert i.has_any_symbols(x) == False

    assert (i*x**i).has_any_symbols(x) == True
    assert (i*y**i).has_any_symbols(x) == False
    assert (i*y**i).has_any_symbols(x, y) == True

    expr = x**2*y + sin(2**t + log(z))

    assert expr.has_any_symbols(u) == False

    assert expr.has_any_symbols(x) == True
    assert expr.has_any_symbols(y) == True
    assert expr.has_any_symbols(z) == True
    assert expr.has_any_symbols(t) == True

    assert expr.has_any_symbols(x, y, z, t) == True
    assert expr.has_any_symbols(x, y, z, t, u)  == True

def test_has_all_symbols():
    x,y,z,t,u = symbols('xyztu')

    i = Integer(4400)

    assert i.has_all_symbols(x) == False

    assert (i*x**i).has_all_symbols(x) == True
    assert (i*y**i).has_all_symbols(x) == False

    expr = x**2*y + sin(2**t + log(z))

    assert expr.has_all_symbols(y, z, t) == True
    assert expr.has_all_symbols(x, z, t) == True
    assert expr.has_all_symbols(x, y, t) == True
    assert expr.has_all_symbols(x, y, z) == True

    assert expr.has_all_symbols(y, u, t) == False
    assert expr.has_all_symbols(x, z, u) == False
    assert expr.has_all_symbols(u, y, z) == False

    assert expr.has_all_symbols(x, y, z, t) == True
    assert expr.has_all_symbols(x, y, z, t, u) == False

def test_as_poly_basic():
    x, y = symbols('xy')

    f = x**2 + 2*x*y

    assert f.as_poly(x, y).as_basic() == f
    assert (f + sin(x)).as_poly(x, y) is None

def test_nonzero():
    assert bool(S.Zero) == False
    assert bool(S.One)  == True
    assert bool(x)      == True
    assert bool(x+y)    == True
    assert bool(x-x)    == False
    assert bool(x*y)    == True
    assert bool(x*1)    == True
    assert bool(x*0)    == False

def test_is_number():
    x, y = symbols('xy')
    g = WildFunction('g')

    assert Real(3.14).is_number == True
    assert Integer(737).is_number == True
    assert Rational(3, 2).is_number == True

    assert x.is_number == False
    assert (2*x).is_number == False

    assert (x + y).is_number == False

    assert log(2).is_number == True
    assert log(x).is_number == False

    assert (2 + log(2)).is_number == True
    assert (2 + log(x)).is_number == False

    assert (2*g).is_number == False



"""
This module implements standard mathematical functions
(exp, log, sin, ...) for real arguments. Complex versions will
be provied in a complementary file cfunctions.py.
"""

from float_ import Float, ComplexFloat
from constants import pi_float, log2_fixed, log2_float, log10_float
from utils_ import bitcount, make_fixed
from math import log as _clog

def _quadratic_steps(start, target):
    L = [target]
    while L[-1] > start*2:
        L = L + [L[-1]//2 + 1]
    return L[::-1]

def _rshift(x, n):
    if n >= 0: return x >> n
    else:      return x << (-n)

def _lshift(x, n):
    if n >= 0: return x << n
    else:      return x >> (-n)

def _with_extraprec(n, f):
    Float._prec += n
    v = f()
    Float._prec -= n
    return +v


#----------------------------------------------------------------------
# Square root
#

"""
Square roots are most efficiently computed with Newton's method.
Two functions are implemented: _sqrt_fixed and _sqrt_fixed2.

  _sqrt_fixed uses the iteration r_{n+1} = (r_n + y/r_n)/2,
  which is just Newton's method applied to the equation r**2 = y.

  _sqrt_fixed2 uses the iteration r_{n+1} = r_n*(3 - y*r_n**2)
  to calculate 1/sqrt(y), and then multiplies by y to obtain
  sqrt(y).

The first iteration is slightly faster at low precision levels, since
it essentially just requires one division at each step, compared to
the three multiplications in the second formula. However, the second
iteration is much better at extremely high precision levels. This is
due to the fact that Python uses the Karatsuba algorithm for integer
multiplication, which is asymptotically faster than its division
algorithm.

For optimal speed, we exploit the "self-correcting" nature of
Newton's method to perform subcomputations at as low a precision level
as possible. Starting from a 50-bit floating-point estimate, the
first step can be computed using 100-bit precision, the second
at 200-bit precision, and so on; full precision is only needed for
the final step.

Both functions use fixed-point arithmetic and assume that the input y
is a big integer, i.e. given the integer y and precision prec,
they return floor(sqrt(x) * 2**prec) where y = floor(x * 2**prec).

The functions currently assume that x ~= 1. (TODO: make the code
work for x of arbitrary magnitude.) The main sqrt() function
fiddles with the exponent of the input to reduce it to unit
magnitude before passing it to _sqrt_fixed or _sqrt_fixed2.

"""

def _sqrt_fixed(y, prec):
    # get 50-bit initial guess from regular float math
    if prec < 200:
        r = int(y**0.5 * 2.0**(50-prec*0.5))
    else:
        r = int((y >> (prec-100))**0.5)
    prevp = 50
    for p in _quadratic_steps(50, prec+8):
        # Newton iteration: r_{n+1} = (r_{n} + y/r_{n})/2
        # print "sqrt", p
        r = _lshift(r, p-prevp-1) + (_rshift(y, prec-p-prevp+1)//r)
        prevp = p
    return r >> 8

def _sqrt_fixed2(y, prec):
    r = float(Float((y, -prec), 64))**-0.5
    r = int(r * 2**50)
    prevp = 50
    for p in _quadratic_steps(50, prec+8):
        # print "sqrt", p
        r2 = _rshift(r*r, 2*prevp - p)
        A = _lshift(r, p-prevp)
        T = _rshift(y, prec-p)
        S = (T*r2) >> p
        B = (3 << p) - S
        r = (A*B)>>(p+1)
        prevp = p
    r = (r * y) >> prec
    return r >> 8

def sqrt(x):
    """
    If x is a positive Float, sqrt(x) returns the square root of x as a
    Float, rounded to the current working precision. If x is negative
    or a ComplexFloat, it returns the principal square root of x
    as a ComplexFloat.
    """
    if isinstance(x, (complex, ComplexFloat)):
        return _csqrt(x)
    if not isinstance(x, Float):
        x = Float(x)
    if x == 0:
        return Float(0)
    if x < 0:
        return _csqrt(ComplexFloat(x,0))
    prec = Float._prec + 4
    # Convert to a fixed-point number with prec bits. Adjust
    # exponents to be even so that they can be divided in half
    if prec & 1: prec += 1
    man = x.man
    exp = x.exp
    if exp & 1:
        exp -= 1
        man <<= 1
    shift = bitcount(man) - prec
    shift -= shift & 1
    man = _rshift(man, shift)
    if prec < 65000:
        man = _sqrt_fixed(man, prec)
    else:
        man = _sqrt_fixed2(man, prec)
    return Float((man, (exp+shift-prec)//2))

"""
For complex square roots, we have

  sqrt(a+b*I) = sqrt((r+a)/2) + I*b/sqrt(2*(r+a))

where r = abs(a+b*I), when a+b*I is not a negative real number.
(http://en.wikipedia.org/wiki/Square_root)
"""

def _csqrt(z):
    if not isinstance(z, ComplexFloat):
        z = ComplexFloat(z)
    Float._prec += 4
    if z == 0:
        return z
    if z.real < 0 and z.imag == 0:
        w = ComplexFloat(0, sqrt(-z.real))
    else:
        r = abs(z)
        rpx = r + z.real
        re = sqrt(rpx/2)
        im = z.imag / sqrt(2*rpx)
        w = ComplexFloat(re, im)
    Float._prec -= 4
    return +w

def hypot(x, y):
    """hypot(x, y) computes the distance function sqrt(x**2+y**2)"""
    if not isinstance(x, Float): x = Float(x)
    if not isinstance(y, Float): y = Float(y)
    if not y: return abs(x)
    if not x: return abs(y)
    return _with_extraprec(4, lambda: sqrt(x*x + y*y))


#----------------------------------------------------------------------
# Exponential function
#

"""
The exponential function has a rapidly convergent Maclaurin series:

    exp(x) = 1 + x + x**2/2! + x**3/3! + x**4/4! + ...

The series can be summed very easily using fixed-point arithmetic.
The convergence can be improved further, using a trick due to
Richard P. Brent: instead of computing exp(x) directly, we choose a
small integer r (say, r=10) and compute exp(x/2**r)**(2**r).

The optimal value for r depends on the Python platform, the magnitude
of x and the target precision, and has to be estimated from
experimental timings. One test with x ~= 0.3 showed that
r = 2.2*prec**0.42 gave a good fit to the optimal values for r for
prec between 1 and 10000 bits, on one particular machine.

This optimization makes the summation about twice as fast at
low precision levels and much faster at high precision
(roughly five times faster at 1000 decimal digits).

If |x| is very large, we first rewrite it as t + n*log(2) with the
integer n chosen such that |t| <= log(2), and then calculate
exp(x) as exp(t)*(2**n), using the Maclaurin series for exp(t)
(the multiplication by 2**n just amounts to shifting the exponent).
"""

def _exp_series(x, prec):
    r = int(2.2 * prec ** 0.42)
    # XXX: more careful calculation of guard bits
    guards = r + 3
    if prec > 60:
        guards += int(_clog(prec))
    prec2 = prec + guards
    x = _rshift(x, r - guards)
    s = (1 << prec2) + x
    a = x
    k = 2
    # Sum exp(x/2**r)
    while 1:
        a = ((a*x) >> prec2) // k
        if not a: break
        s += a
        k += 1
    # Calculate s**(2**r) by repeated squaring
    for j in range(r):
        s = (s*s) >> prec2
    return s >> guards

def exp(x):
    """
    exp(x) -- compute the exponential function of the real or complex
    number x
    """
    if isinstance(x, (ComplexFloat, complex)):
        mag = exp(x.real)
        re, im = cos_sin(x.imag)
        return ComplexFloat(mag*re, mag*im)
    else:
        if not isinstance(x, Float):
            x = Float(x)
        # extra precision needs to be similar in magnitude to log_2(|x|)
        prec = Float._prec + 4 + max(0, bitcount(x.man) + x.exp)
        t = make_fixed(x, prec)
        if abs(x) > 1:
            lg2 = log2_fixed(prec)
            n, t = divmod(t, lg2)
        else:
            n = 0
        y = _exp_series(t, prec)
        return Float((y, -prec+n))


#----------------------------------------------------------------------
# Natural logarithm
#

"""
The basic strategy for computing log(x) is to set r = log(x) and use
Newton's method to solve the equation exp(r) = x. We set the initial
value r_0 to math.log(x) and then iterate r_{n+1} = r_n + exp(-r_n) - 1
until convergence. As with square roots, we increase the working
precision dynamically during the process so that only one full-precision
evaluation of exp is required.

log(x) is small for most inputs, so the r values can safely be
computed using fixed-point arithmetic. However, when x has a very
large or small exponent, we can improve performance through the
normalization log(t * 2**n) = log(t) + n*log(2), choosing n such
that 0.5 <= t <= 1 (for example).

There are some caveats: if x is extremely close to 1, the working
precision must be increased to maintain high relative precision in the
output (alternatively, the series approximation for log(1+x) could
be used in that case).
"""

# This function performs the Newton iteration using fixed-point
# arithmetic. x is assumed to have magnitude ~= 1
def _log_newton(x, prec):
    # 50-bit approximation
    r = int(_clog(Float((x, -prec), 64)) * 2.0**50)
    prevp = 50
    for p in _quadratic_steps(50, prec+8):
        rb = _lshift(r, p-prevp)
        e = _exp_series(-rb, p)
        r = rb + ((_rshift(x, prec-p)*e)>>p) - (1 << p)
        prevp = p
    return r >> 8

def log(x, b=None):
    """
    log(x)    -> the natural (base e) logarithm of x
    log(x, b) -> the base b logarithm of x

    """
    # Basic input management
    if b is not None:
        Float._prec += 3
        if   b == 2:  blog = log2_float()
        elif b == 10: blog = log10_float()
        else: blog = log(b)
        l = log(x) / blog
        Float._prec -= 3
        return l

    if x == 0:
        raise ValueError, "logarithm of 0"

    if isinstance(x, (ComplexFloat, complex)):
        mag = abs(x)
        phase = atan2(x.imag, x.real)
        return ComplexFloat(log(mag), phase)

    if not isinstance(x, Float):
        x = Float(x)

    if x < 0:
        return log(ComplexFloat(x))

    if x == 1: return Float((0, 0))

    bc = bitcount(x.man)
    # Estimated precision needed for log(t) + n*log(2)
    prec = Float._prec + int(_clog(1+abs(bc+x.exp), 2)) + 10

    # Watch out for the case when x is very close to 1
    if -1 < bc+x.exp < 2:
        near_one = abs(x-1)
        if near_one == 0:
            return Float((0, 0))
        prec += -(near_one.exp) - bitcount(near_one.man)

    # Separate mantissa and exponent, calculate fixed-point
    # approximation and put it all together
    t = _rshift(x.man, bc-prec)
    l = _log_newton(t, prec)
    a = (x.exp+bc) * log2_fixed(prec)
    return Float((l+a, -prec))


# XXX: need to increase working precision here
def power(x, y):
    return exp(log(x) * y)


#----------------------------------------------------------------------
# Trigonometric functions
#

"""
We compute sin(x) around 0 from its Taylor series, and cos(x) around 0
from sqrt(1-sin(x)**2). This way we can simultaneously compute sin and
cos, which are often needed together (e.g. for the tangent function or
the complex exponential), with little extra cost compared to computing
just one of them. The main reason for computing sin first (and not cos
from sin) is to obtain high relative accuracy for x extremely close to
0, where the operation sqrt(1-cos(x)**2) can cause huge cancellations.

For any value of x, we can reduce it to the interval A = [-pi/4, pi/4]
(where the Taylor series converges quickly) by translations, changing
signs, and switching the roles of cos and sin:

   A : sin(x) = sin(x)           cos(x) = cos(x)
   B : sin(x) = cos(x-pi/2)      cos(x) = -sin(x-pi/2)
   C : sin(x) = -sin(x-pi)       cos(x) = -cos(x-pi)
   D : sin(x) = -cos(x-3*pi/2)   cos(x) = sin(x-3*pi/2)

|     A      |      B     |      C     |     D     |
v            v            v            v           v

   1 |  ____   ..........                            ____
     |      _..          ..                        __
     |      . __           .                     __
     |    ..    _           ..                  _
     |   .       __           .               __
-----| -.----------_-----------.-------------_-----------
     | .            _           ..          _           .
     |               __           .       __           .
     |                 _           ..    _           ..
     |                  __           . __           .
     |                    __         _..          ..
  -1 |                      _________   ..........
      0                       pi                     2*pi

TODO: check if the reduced x is very close to 0 and in that case repeat
the mod operation at a higher level of precision.
"""

from constants import pi_fixed

def _sin_series(x, prec):
    x2 = (x*x) >> prec
    s = a = x
    k = 3
    while a:
        a = ((a * x2) >> prec) // (-k*(k-1))
        s += a
        k += 2
    return s

def _trig_reduce(x, prec):
    pi_ = pi_fixed(prec)
    pi4 = pi_ >> 2
    pi2 = pi_ >> 1
    n, rem = divmod(x + pi4, pi2)
    rem -= pi4
    return n, rem

def cos_sin(x):
    """
    cos_sin(x) calculates both the cosine and the sine of x rounded
    to the nearest Float value, and returns the tuple (cos(x), sin(x)).
    """
    if not isinstance(x, Float):
        x = Float(x)
    bits_from_unit = abs(bitcount(x.man) + x.exp)
    prec = Float._prec + bits_from_unit + 15
    xf = make_fixed(x, prec)
    n, rx = _trig_reduce(xf, prec)
    case = n % 4
    one = 1<<prec
    if case == 0:
        s = _sin_series(rx, prec)
        c = _sqrt_fixed(one - ((s*s)>>prec), prec)
    elif case == 1:
        c = -_sin_series(rx, prec)
        s = _sqrt_fixed(one - ((c*c)>>prec), prec)
    elif case == 2:
        s = -_sin_series(rx, prec)
        c = -_sqrt_fixed(one - ((s*s)>>prec), prec)
    elif case == 3:
        c = _sin_series(rx, prec)
        s = -_sqrt_fixed(one - ((c*c)>>prec), prec)
    return Float((c, -prec)), Float((s, -prec))

def cos(x):
    return cos_sin(x)[0]

def sin(x):
    if isinstance(x, (ComplexFloat, complex)):
        c, s = cos_sin(x.real)
        expb1 = exp(x.imag)
        expb2 = 1 / expb1
        ch = (expb1 + expb2) / 2
        sh = (expb1 - expb2) / 2
        return ComplexFloat(s*ch, c*sh)
    else:
        return cos_sin(x)[1]

def tan(x):
    Float._prec += 2
    c, s = cos_sin(x)
    t = s / c
    Float._prec -= 2
    return t


#----------------------------------------------------------------------
# Inverse tangent
#

"""
Near x = 0, use atan(x) = x - x**3/3 + x**5/5 - ...
Near x = 1, use atan(x) = y/x * (1 + 2/3*y + 2*4/3/5*y**2 + ...)
where y = x**2/(1+x**2).

TODO: these series are not impressively fast. It is probably better
to calculate atan from tan, using Newton's method.

"""

def _atan_series_1(x):
    prec = Float._prec
    # Increase absolute precision when extremely close to 0
    bc = bitcount(x.man)
    diff = -(bc + x.exp)
    if diff > 10:
        if 3*diff - 4 > prec: # x**3 term vanishes; atan(x) ~x
            return +x
        prec = prec + diff
    prec += 15 # XXX: better estimate for number of guard bits
    x = make_fixed(x, prec)
    x2 = (x*x)>>prec; one = 1<<prec; s=a=x
    for n in xrange(1, 1000000):
        a = (a*x2) >> prec
        s += a // ((-1)**n * (n+n+1))
        if -100 < a < 100:
            break
    return Float((s, -prec))

def _atan_series_2(x):
    prec = Float._prec
    prec = prec + 15
    x = make_fixed(x, prec)
    one = 1<<prec; x2 = (x*x)>>prec; y=(x2<<prec)//(one+x2)
    s = a = one
    for n in xrange(1, 1000000):
        a = ((a*y)>>prec) * (2*n) // (2*n+1)
        if a < 100: break
        s += a
    return Float(((y*s)//x, -prec))

def atan(x):
    """Compute atan(x) for a real number x"""
    if not isinstance(x, Float): x = Float(x)
    if x < -0.6: return _with_extraprec(2, lambda: -atan(-x))
    if x < 0.6: return _atan_series_1(x)
    if x < 1.5: return _atan_series_2(x)
    # For large x, use atan(x) = pi/2 - atan(1/x)
    Float._prec += 4
    prec = Float._prec
    if x.exp > 10*prec:
        t = pi_float()/2   # XXX
    else:
        t = pi_float()/2 - atan(Float(1)/x)
    Float._prec -= 4
    return +t

def atan2(y,x):
    """atan2(y,x) has the same magnitude as atan(y/x) but
    accounts for the signs of y and x"""
    if y < 0: return -atan2(-y, x)
    if not x and not y: return Float(0)
    if y > 0 and x == 0:
        Float._prec += 2; t = pi_float()/2; Float._prec -= 2
        return t
    Float._prec += 2
    if x > 0: a = atan(y/x)
    else:     a = pi_float() - atan(-y/x)
    Float._prec -= 2
    return +a




from sympy import Basic, S, C, Symbol, Mul, I, pi, exp, sqrt
from sympy.core import sympify
from sympy.core.numbers import Rational, Integer, gcd
from sympy.core.methods import NoRelMeths, ArithMeths
from sympy.polynomials import quo, factor_, PolynomialException
from sympy.numerics.optimize import polyroots

def linear(a, b):
    return [-b/a]

def quadratic(a, b, c):
    d = (b**2 - 4*a*c)**S.Half

    R = [(-b + d) / (2*a),
         (-b - d) / (2*a)]

    return [ r.expand() for r in R ]

def cubic(a, b, c, d):
    p = (3*c/a - (b/a)**2) / 9
    q = (9*b*c/a**2 - 27*d/a - 2*(b/a)**3) / 54

    d = (p**3 + q**2)**S.Half

    s = (q - d)**Rational(1,3)
    t = (q - d)**Rational(1,3)

    I, J = s + t, s - t

    u, v = b/(3*a), I*sqrt(3)/2

    R = [I - u,
         -I/2 - u + v*J,
         -I/2 - u - v*J]

    return [ r.expand() for r in R ]

def quartic(a, b, c, d, e):
    p = -2*b
    q = b**2 + a*c - 4*d
    r = c**2 + a**2*d - a*b*c

    r = cubic(1, p, q, r)

    u = r[1] + r[2] - r[0]
    v = (u**2 - 16*d)**S.Half

    w = (a**2 - 4*r[0])**S.Half

    A = (u + v) / 4
    B = (u - v) / 4
    C = (-a + w) / 2
    D = (-a - w) / 2

    E = (C**2 - 4*A)**S.Half
    F = (D**2 - 4*B)**S.Half

    R = [(C + E) / 2, (C - E) / 2,
         (D + F) / 2, (D - F) / 2]

    return [ r.expand() for r in R ]

def arbitrary(poly):
    g = reduce(gcd, [ int(e) for _, e in poly.coeffs ])

    if g in [0, 1]:
        return []

    n = int(poly.degree() / g)

    if n not in [1, 2, 3]:
        return []

    poly = C.Polynomial(coeffs=[ (c, e/g) for c, e in poly.coeffs ],
        var=poly.var, order=poly.order)

    roots_of_unity = [ exp(2*i*pi*I/g) for i in range(g) ]

    solutions = []

    for zeta in roots_of_unity:
        zeta = zeta.expand(complex=True)

        for r in roots(poly):
            R = r**Rational(1,g)

            if not r.atoms(Symbol):
                R = R.expand(complex=True)

            solutions.append((zeta * R).expand())

    return solutions

def roots(poly, x=None, domain=None, **kwargs):
    if not isinstance(poly, C.Polynomial):
        try:
            poly = poly.as_polynomial(x)
        except PolynomialException:
            return []

    n = int(poly.degree())

    if n == 0:
        return []

    if domain == 'F':
        if len(poly.atoms(Symbol)) == 1:
            return sorted([ sympify(complex(r)) \
                for r in polyroots(poly.as_basic())[0] ])
        else:
            raise ValueError("'F' domain requires numeric coefficients")

    roots = arbitrary(poly)

    if roots == []:
        factors = [ poly ]

        if n < 5 or kwargs.get('factor', False):
            try:
                factors = factor_.factor(poly)
            except PolynomialException:
                pass

        handlers = {
            1 : linear,
            2 : quadratic,
            3 : cubic,
            4 : quartic
        }

        for factor in factors:
            n = int(factor.degree())

            if n == 0:
                continue

            if n in handlers:
                coeffs = [ factor.nth_coeff(i) for i in range(n+1) ]
                roots += handlers[n](*reversed(coeffs))
            else:
                roots += arbitrary(factor)

    Zr, Qr, Rr, Ir, Cr = [], [], [], [], []

    for root in roots:
        if root.is_Integer:
            Zr.append(root)
        elif root.is_Rational:
            Qr.append(root)
        elif root.is_real:
            Rr.append(root)
        elif root.is_imaginary:
            Ir.append(root)
        else:
            Cr.append(root)

    Zr, Qr, Rr = map(sorted, (Zr, Qr, Rr))

    domains = {
        'Z' : Zr,
        'Q' : Zr+Qr,
        'R' : Zr+Qr+Rr,
        'I' : Ir,
        'C' : Zr+Qr+Rr+Ir+Cr
    }

    if domain is None:
        roots = domains['C']
    else:
        try:
            roots = domains[domain]
        except KeyError:
            raise ValueError("Invalid domain: %s" % domain)

    return roots

def factors(poly, x=None, domain=None, **kwargs):
    _roots = roots(poly, x, domain, **kwargs)

    if isinstance(poly, C.Polynomial):
        poly = poly.as_basic()

    if not _roots:
        return [ poly ]
    else:
        factored = [ x-r for r in _roots ]
        f = quo(poly, Mul(*factored), x)

        if f.has(x):
            return factored + [ f ]
        else:
            return factored

_exact_roots_cache = {}
_float_roots_cache = {}

def _exact_roots(poly):
    try:
        exact = _exact_roots_cache[poly]
    except KeyError:
        exact = roots(poly, domain='C')
        _exact_roots_cache[poly] = exact

    return exact

def _float_roots(poly):
    try:
        approx = _float_roots_cache[poly]
    except KeyError:
        approx = roots(poly, domain='F')
        _float_roots_cache[poly] = approx

    return approx

class RootOf(Basic, NoRelMeths, ArithMeths):
    """Represents all roots or a specific root of a polynomial.

       RootOf class is dual to support handling of all roots
       of a given polynomial or just a single root specified
       using integer index. In both cases roots can be computed
       exactly, if possible, or using numerical approximation.

       >>> from sympy import *
       >>> x, y = symbols('xy')

       >>> prec = Basic.set_precision(6)

       >>> RootOf(2 + x**2, x).count()
       2

       >>> RootOf(2 + x**2, x, 0)
       I*2**(1/2)
       >>> RootOf(2 + x**2, x, 1)
       -I*2**(1/2)

       >>> list(RootOf(2 + x**2, x).roots())
       [I*2**(1/2), -I*2**(1/2)]

       >>> roots = list(RootOf(1 + x + x**7, x).roots())
       >>> print roots[0]
       RootOf(1 + x + x**7, x, 0)
       >>> print roots[1]
       RootOf(1 + x + x**7, x, 1)

       and so on ...

       >>> RootOf(1 + x + x**7, x, 1).evalf()
       -0.705298 - 0.637624*I

       >>> _ = Basic.set_precision(prec)

    """

    def __new__(cls, poly, x, index=None, **assumptions):
        try:
            poly = poly.as_polynomial(x)
        except PolynomialException:
            raise TypeError("'RootOf' operates on polynomials")

        if index is not None:
            if index < 0 or index >= poly.degree():
                raise IndexError("Index must be in "
                    "[0, %d] range" % int(poly.degree()-1))
            else:
                exact = _exact_roots(poly)

                if index < len(exact):
                    return exact[index]

        obj = Basic.__new__(cls, **assumptions)
        obj._args = (poly, x, index)

        return obj

    @property
    def poly(self):
        return self._args[0]

    @property
    def x(self):
        return self._args[1]

    @property
    def index(self):
        return self._args[2]

    @property
    def represents_all(self):
        return self.index is None

    @property
    def represents_one(self):
        return self.index is not None

    def tostr(self, level=0):
        poly, x = (self.poly.tostr(), self.x.tostr())

        if self.represents_one:
            return "RootOf(%s, %s, %d)" % (poly, x, self.index)
        else:
            return "RootOf(%s, %s)" % (poly, x)

    def count(self):
        if self.represents_one:
            return 1
        else:
            return int(self.poly.degree())

    def roots(self):
        if self.represents_one:
            yield self
        else:
            exact = _exact_roots(self.poly)

            for i, root in enumerate(exact):
                yield root

            for i in range(len(exact), self.count()):
                yield RootOf(self.poly, self.x, i)

    def _eval_evalf(self):
        if self.represents_one:
            approx = _float_roots(self.poly)
            return approx[self.index]
        else:
            raise TypeError("Use 'approx' method to obtain all roots")

    def approx(self):
        if self.represents_one:
            yield self.evalf()
        else:
            approx = _float_roots(self.poly)

            for root in approx:
                yield root

    def atoms(self, *args, **kwargs):
        return self.poly.atoms(*args, **kwargs)



from sympy.core.symbol import Symbol
from sympy.polynomials import Polynomial
from float_ import Float, ComplexFloat
from evalf_ import polyfunc
from utils_ import bitcount
from functions import *

import random
rng = random.Random()

class ConvergenceError(Exception):
    pass


def polyroots(poly, maxsteps=20):
    """
    Numerically locate all (complex) roots of a given polynomial 'poly'
    using the Durand-Kerner method (http://en.wikipedia.org/wiki/
    Durand-Kerner_method)

    This function returns a tuple (roots, err) where roots is a list of
    ComplexFloats sorted by absolute value, and err is an estimate of
    the maximum error. The 'poly' argument should be a SymPy expression
    representing a univariate polynomial.

    Example:
        >>> Float.setdps(15)
        >>> x = Symbol('x')
        >>> r, e = polyroots((x-3)*(x-2))
        >>> r[0]
        ComplexFloat(real='1.9999999999999993', imag='-4.5917748078995606E-41')
        >>> r[1]
        ComplexFloat(real='3', imag='-4.6222318665293660E-33')
        >>> e
        Float('2.2204460492503131E-16')

    Polyroots attempts to achieve a maximum error less than the epsilon
    of the current working precision, but may fail to do so if the
    polynomial is badly conditioned. Usually the error can be reduced
    by increasing the 'maxsteps' parameter, although this will of
    course also reduce the speed proprtionally. It may also be
    necessary to increase the working precision.

    In particular, polyroots is easily fooled by multiple roots and
    roots that are very close together. In general, if n-multiple roots
    are present, polyroots will typically only locate their values to
    within 1/n of the working precision. For example, with the standard
    precision of ~15 decimal digits, the expected accuracy for a double
    root is 7-8 decimals:

        >>> Float.setdps(15)
        >>> r, e = polyroots((x-2)**2)
        >>> r[0]
        ComplexFloat(real='1.9999999832018203', imag='-2.5469994476319085E-8')
        >>> r[1]
        ComplexFloat(real='2.0000000110216827', imag='1.4784776969781909E-8')
        >>> e
        Float('5.4076851354766810E-8')

    An effective cure is to multiply the working precision n times (in
    the case of a double root, doubling it):

        >>> Float.setdps(30)
        >>> r, e = polyroots((x-2)**2, 40)
        >>> r[0].real
        Float('1.9999999999999999075008441778756')
        >>> e
        Float('1.9055980809840526328094566161453E-16')

    The result is good to 15 decimals as expected.
    """
    # Must be monic
    poly = Polynomial(poly)
    poly = Polynomial(poly / poly.coeffs[0][0])
    deg = poly.coeffs[0][1]
    f = polyfunc(poly)
    roots = [ComplexFloat(0.4+0.9j)**n for n in range(deg)]
    error = [Float(1) for n in range(deg)]
    for step in range(maxsteps):
        if max(error).ae(0):
            break
        for i in range(deg):
            if not error[i].ae(0):
                p = roots[i]
                x = f(p)
                for j in range(deg):
                    if i != j:
                        try:
                            x /= (p-roots[j])
                        except ZeroDivisionError:
                            continue
                roots[i] = p - x
                error[i] = abs(x)
    roots.sort(key=abs)
    err = max(error)
    err = max(err, Float((1, -Float._prec+1)))
    return roots, err


def sign(x):
    if x < 0:
        return -1
    return 1


def bisect(f, a, b, eps=None, maxsteps=None, verbose=False):
    """
    Numerical root-finding using the bisection method.

    Given a real-valued function f and an interval [a, b] (not
    necessarily real) such that f(a) and f(b) have opposite signs,
    narrow the interval where f crosses the x axis to a relative
    width at most equal to 'eps' through repeated bisections. If
    not specified, eps is set to give a full precision estimate
    of the root.

    A tuple for the narrowed interval is returned.

    If f is continuous, bisect is guaranteed to find a root. If f
    jumps discontinuously from positive negative values, bisect will
    locate the point of the discontinuity. bisect quits early and
    returns an interval of width zero if it encounters a point x
    where f(x) = 0 exactly.

    Optionally, perform no more than 'maxsteps' bisections (by
    default perform as many as needed); if the 'verbose' flag is set,
    print status at each step.

    Examples
    ========

    Find a bounding interval for pi/2 (which is a root of cos):

        >>> Float.setdps(15)
        >>> a, b = bisect(cos, 1, 2, 1e-2, verbose=True)
          bisect step  1: a=1.000000  b=2.000000  delta=1
          bisect step  2: a=1.500000  b=2.000000  delta=0.5
          bisect step  3: a=1.500000  b=1.750000  delta=0.25
          bisect step  4: a=1.500000  b=1.625000  delta=0.125
          bisect step  5: a=1.562500  b=1.625000  delta=0.0625
          bisect step  6: a=1.562500  b=1.593750  delta=0.03125
        >>> print a, b
        1.5625 1.578125

    Calculate the same value to full precision:

        >>> a, b = bisect(cos, 1, 2)
        >>> a, b
        (Float('1.5707963267948966'), Float('1.5707963267948968'))
        >>> print cos(a)
        6.12320117570134E-17
        >>> print cos(b)
        -1.60812593168018E-16

    Although generally reliable, the bisection method has a slow rate
    of convergence. The previous computation required about 50 steps,
    which can be compared to the secant method which only requires
    5-6 steps to obtain a full precision value from an initial value
    near 1.5.
    """

    a, b = a*Float(1), b*Float(1)   # Convert to Float or ComplexFloat
    fa0, fb0 = f(a), f(b)

    # Sanity check
    if not fa0 * fb0 <= 0:
        raise ValueError("bisect: f(a) and f(b) have the same sign at a=%r "
            " and b=%r" %  (a, b))

    fa0sign = sign(fa0)

    # Default eps / set eps to something sane if zero
    mineps = Float((1, -Float.getprec()+1))
    if not eps or eps < mineps:
        eps = mineps

    if maxsteps is None:
        # Use maxsteps as a safeguard should the loop condition fail
        abdelta = abs(a-b)
        maxsteps = bitcount(abdelta.man) - abdelta.exp + Float.getprec()

    i = 1
    while not a.ae(b, eps) and i <= maxsteps:
        if verbose:
            print "  bisect step %2i: a=%8f  b=%8f  delta=%g" % (i,a,b,b-a)
        mid = (a+b)*0.5
        fmid = f(mid)
        if fmid == 0:
            return (mid, mid)
        if sign(fmid) == fa0sign:
            a = mid
        else:
            b = mid
        i += 1

    return a, b


def perturb(x, eps):
    #rng.seed(x)
    if x == 0:
        return eps
    else:
        #return x * (1 + rng.random()*eps)
        return x * (1 + eps)


def secant(f, x0, x1=None, eps=None, maxsteps=50, bracket=None,
    on_error=1, verbose=False):
    """
    secant(f, x0) uses the secant method to find a numerical root of f,
    starting from the initial approximation/guess x0.

    This algorithm is very efficient for finding regular roots of
    smooth functions; nearly as efficient as Newton's method. Each
    iteration roughly multiplies the number of correct digits in the
    answer by 1.6 (in practice, a 15-digit value can usually be
    found in less than 10 steps).

    The secant method can be inefficient or may fail to converge in
    the presence of high-order roots or if the initial approximation
    is far from the true root. Generally, it works best if f is
    monotonic and steeply sloped everywhere in the surrounding of
    the root (and x0 is in that surrounding).

    Advanced settings
    =================

    The secant method actually requires two initial values x0 and x1.
    By default, an x1 value is generated by perturbing x0 slightly;
    in some cases this perturbation may be too small and it may then
    help to specify a custom value for x1.

    Several other advanced options are supported as guards against
    poor convergence:

      eps
          Only attempt to locate the root to within this epsilon.
          By default, eps is set to give a full-precision value.

      maxsteps
          Break when this many iterations have been performed.

      bracket
          If bracket = [a, b], break if the iteration ends up somewhere
          outside the interval [a, b]. This is useful to protect from
          the iterate x_n "shooting off to space" in the presence of
          a nearly horizontal function value. If the bracket is a
          single number, break if abs(x_0-x_n) exceeds this value.

    The 'on_error' parameter can be set to control what happens in the
    case of a convergence failure:

       on_error = 0   print warning if in verbose mode; return value
       on_error = 1   print warning; return value (default)
       on_error = 2   raise an exception

    Examples
    ========

    A simple example, calculating pi:

        >>> Float.setdps(15)
        >>> print secant(sin, 3)
        3.14159265358979

    If we try the same for cosine, starting from x=0, we get the
    "wrong" root because the cosine is nearly horizontal around x=0,
    sending the initial iterate far away (verbose=True shows what
    happens in more detail):

        >>> Float.setdps(15)
        >>> print secant(cos, 0)
        391.128285371929

    This can be helped by providing a second point that better matches
    the geometry of the function's graph or by providing a closer
    initial estimate:

        >>> print secant(cos, x0=0, x1=1)
        1.57079632679490
        >>> print secant(cos, 1.5)
        1.57079632679490

    As another example, a high-precision calculation of log(3):

        >>> Float.setdps(50)
        >>> print secant(lambda x: exp(x)-3, 1)
        1.0986122886681096913952452369225257046474905578227
        >>> Float.revert()
    """

    prec = Float.getprec()
    if eps is None:
        eps = Float((1, -prec+2))
    else:
        eps = Float(eps)

    x = x0 = x0 * Float(1)

    if x1 is None:
        # Perturb the initial value to generate a second point as
        # needed to start the secant method iteration
        xprev = perturb(x, 0.01)
    else:
        xprev = x1 * Float(1)

    if isinstance(bracket, (list, tuple)):
        brackets = Float(bracket[0]), Float(bracket[1])
    else:
        brackets = None

    exception = None
    bracketmsg = "Failed to converge (outside bracket)"
    stepsmsg = "Failed to converge (maxsteps exceeded)"
    derivmsg = "Failed to converge (inft./zero derivative or " \
        "loss of precision in derivative)"

    for i in xrange(1, maxsteps+1):
        if verbose:
            try:
                print "  secant: x_%i=%8f  delta=%g" % (i,x,x-xprev)
            except TypeError:
                print "  secant: x_%i=%s  delta=%s" % (i,x,x-xprev)

        # Calculate function values
        fx = f(x)
        fdiff = fx - f(xprev)
        xdiff = x - xprev

        if xdiff.ae(0, eps):
            break

        # Try to calculate the finite difference approximation for the
        # derivative and update accordingly. In floating-point
        # arithmetic, this may cause a division by zero when f(x) is
        # extremely close to 0, which we have to watch out for
        try:
            deriv = xdiff/fdiff
            x, xprev = x - fx*deriv, x
        except ZeroDivisionError:
            exception = derivmsg
            break

        # Check if within reasonable bounds
        if brackets:
            if not brackets[0] <= x <= brackets[1]:
                exception = bracketmsg
                break
        elif bracket is not None:
            print "heh", x, x0, x-x0, bracket
            if abs(x-x0) >= bracket:
                exception = bracketmsg
                break

    if i == maxsteps:
        exception = stepsmsg

    if exception:
        if on_error == 2: raise ConvergenceError(exception)
        if on_error == 1 or verbose: print exception

    return x




def all(iterable):
    """Return True if all elements are set to True. This
       function does not support predicates explicitely,
       but this behaviour can be simulated easily using
       list comprehension.

       >>> all( [True, True, True] )
       True
       >>> all( [True, False, True] )
       False
       >>> all( [ x % 2 == 0 for x in [2, 6, 8] ] )
       True
       >>> all( [ x % 2 == 0 for x in [2, 6, 7] ] )
       False

       NOTE: Starting from Python 2.5 this a built-in.
    """
    for item in iterable:
        if not item:
            return False
    return True

def any(iterable):
    """Return True if at least one element is set to True.
       This function does not support predicates explicitely,
       but this behaviour can be simulated easily using
       list comprehension.

       >>> any( [False, False, False] )
       False
       >>> any( [False, True, False] )
       True
       >>> any( [ x % 2 == 1 for x in [2, 6, 8] ] )
       False
       >>> any( [ x % 2 == 1 for x in [2, 6, 7] ] )
       True

       NOTE: Starting from Python 2.5 this a built-in.
    """
    for item in iterable:
        if item:
            return True
    return False

def make_list(expr, kind):
    """Returns a list of elements taken from specified expresion
       when it is of sequence type (Add or Mul) or singleton list
       otherwise (Rational, Pow etc.).

       >>> from sympy import *
       >>> x, y = map(Symbol, 'xy')

       >>> make_list(x*y, Mul)
       [x, y]
       >>> make_list(x*y, Add)
       [x*y]
       >>> make_list(x*y + y, Add)
       [y, x*y]

    """
    if isinstance(expr, kind):
        return list(expr.args[:])
    else:
        return [expr]

def flatten(iterable):
    """Recursively denest iterable containers.

       >>> flatten([1, 2, 3])
       [1, 2, 3]
       >>> flatten([1, 2, [3]])
       [1, 2, 3]
       >>> flatten([1, [2, 3], [4, 5]])
       [1, 2, 3, 4, 5]
    """
    result = []

    for item in iterable:
        if isinstance(item, list):
            result.extend(flatten(item))
        else:
            result.append(item)

    return result



from sympy import symbols, lambdify, sqrt, sin, cos, pi
from sympy.utilities.pytest import XFAIL
from sympy import symbols, lambdify, sqrt, sin, cos, pi, atan, Rational, Real
from sympy.thirdparty import mpmath
import math, sympy

# high precision output of sin(0.2*pi) is used to detect if precision is lost unwanted
mpmath.mp.dps = 50
sin02 = mpmath.mpf("0.19866933079506121545941262711838975037020672954020")

x,y,z = symbols('xyz')

#================== Test different arguments ==============
def test_no_args():
    f = lambdify([], 1)
    try:
        f(-1)
        raise Exception()
    except TypeError:
        pass
    assert f() == 1

def test_single_arg():
    f = lambdify(x, 2*x)
    assert f(1) == 2

def test_list_args():
    f = lambdify([x,y], x+y)
    assert f(1,2) == 3

def test_str_args():
    f = lambdify('x,y,z', 'z,y,x')
    assert f(3,2,1) == (1,2,3)
    assert f(1.0,2.0,3.0) == (3.0,2.0,1.0)
    # make sure correct number of args required
    try:
        f(0)
        raise Exception()
    except TypeError:
        pass

def test_own_namespace():
    myfunc = lambda x:1
    f = lambdify(x, sin(x), {"sin":myfunc})
    assert f(0.1) == 1
    assert f(100) == 1

def test_own_module():
    f = lambdify(x, sin(x), math)
    assert f(0)==0.0
    f = lambdify(x, sympy.ceiling(x), math)
    try:
        f(4.5)
        raise Exception
    except NameError:
        pass

def test_bad_args():
    try:
        # no vargs given
        f = lambdify(1)
        raise Exception()
    except TypeError:
        pass
    try:
        # same with vector exprs
        f = lambdify([1,2])
        raise Exception()
    except TypeError:
        pass

#================== Test different modules ================
def test_sympy_lambda():
    f = lambdify(x, sin(x), "sympy")
    assert f(x) is sin(x)
    prec = 1e-15
    assert -prec < f(Rational(1,5)).evalf() - Real(str(sin02)) < prec
    try:
        # arctan is in numpy module and should not be available
        f = lambdify(x, arctan(x), "sympy")
        raise Exception
    except NameError:
        pass

def test_math_lambda():
    f = lambdify(x, sin(x), "math")
    prec = 1e-15
    assert -prec < f(0.2) - sin02 < prec
    try:
        f(x) # if this succeeds, it can't be a python math function
        raise Exception
    except ValueError:
        pass

def test_mpmath_lambda():
    f = lambdify(x, sin(x), "mpmath")
    prec = 1e-49 # mpmath precision is around 50 decimal places
    assert -prec < f(mpmath.mpf("0.2")) - sin02 < prec
    try:
        f(x) # if this succeeds, it can't be a mpmath function
        raise Exception
    except TypeError:
        pass

@XFAIL
def test_number_precision():
    f = lambdify(x, sin02, "mpmath")
    prec = 1e-49 # mpmath precision is around 50 decimal places
    assert -prec < f(0) - sin02 < prec

#================== Test Translations =====================
# We can only check if all translated functions are valid. It has to be checked
# by hand if they are complete.

def test_math_transl():
    from sympy.utilities.lambdify import MATH_TRANSLATIONS
    for sym, mat in MATH_TRANSLATIONS.iteritems():
        assert sym in sympy.functions.__dict__
        assert mat in math.__dict__

def test_mpmath_transl():
    from sympy.utilities.lambdify import MPMATH_TRANSLATIONS
    for sym, mat in MPMATH_TRANSLATIONS.iteritems():
        assert sym in sympy.functions.__dict__
        assert mat in mpmath.__dict__

#================== Test some functions ===================
def test_exponentiation():
    f = lambdify(x, x**2)
    assert f(-1) == 1
    assert f(0) == 0
    assert f(1) == 1
    assert f(-2) == 4
    assert f(2) == 4
    assert f(2.5) == 6.25

def test_sqrt():
    f = lambdify(x, sqrt(x))
    assert f(0) == 0.0
    assert f(1) == 1.0
    assert f(4) == 2.0
    assert abs(f(2) - 1.414) < 0.001
    assert f(6.25) == 2.5
    try:
        f(-1)
        raise Exception()
    except ValueError: pass

def test_trig():
    f = lambdify([x], [cos(x),sin(x)])
    d = f(pi)
    prec = 1e-11
    assert -prec < d[0]+1 < prec
    assert -prec < d[1] < prec
    d = f(3.14159)
    prec = 1e-5
    assert -prec < d[0]+1 < prec
    assert -prec < d[1] < prec

#================== Test vectors ==========================
def test_vector_simple():
    f = lambdify((x,y,z), (z,y,x))
    assert f(3,2,1) == (1,2,3)
    assert f(1.0,2.0,3.0) == (3.0,2.0,1.0)
    # make sure correct number of args required
    try:
        f(0)
        raise Exception()
    except TypeError: pass

def test_vector_discontinuous():
    f = lambdify(x, (-1/x, 1/x))
    try:
        f(0)
        raise Exception()
    except ZeroDivisionError: pass
    assert f(1) == (-1.0, 1.0)
    assert f(2) == (-0.5, 0.5)
    assert f(-2) == (0.5, -0.5)

def test_trig_symbolic():
    f = lambdify([x], [cos(x),sin(x)])
    d = f(pi)
    assert abs(d[0]+1) < 0.0001
    assert abs(d[1]-0) < 0.0001

def test_trig_float():
    f = lambdify([x], [cos(x),sin(x)])
    d = f(3.14159)
    assert abs(d[0]+1) < 0.0001
    assert abs(d[1]-0) < 0.0001

def test_docs():
    f = lambdify(x, x**2)
    assert f(2) == 4
    f = lambdify([x,y,z], [z,y,x])
    assert f(1, 2, 3) == [3, 2, 1]
    f = lambdify(x, sqrt(x))
    assert f(4) == 2.0
    f = lambdify((x,y), sin(x*y)**2)
    assert f(0, 5) == 0

def test_math():
    f = lambdify((x, y), sin(x), modules="math")
    assert f(0, 5) == 0

def test_sin():
    f = lambdify(x, sin(x)**2)
    assert isinstance(f(2), float)
    f = lambdify(x, sin(x)**2, modules="math")
    assert isinstance(f(2), float)



from sympy import Rational, symbols, I, sin, cos, pi
from sympy.simplify.rootof import roots, RootOf

def test_roots():
    a,b,c,x = symbols('abcx')

    assert roots(x**2-3*x+2, x) == [1, 2]
    assert roots(x**2-3*x/2+Rational(1,2), x) == [1, Rational(1,2)]
    assert roots(2*x**2-3*x+1, x) == [1, Rational(1,2)]
    assert roots(x**2-1, x) == [-1, 1]
    assert roots(x**2+1, x) == [I, -I]
    assert roots(x**3-1, x) == [1,
                             Rational(-1,2) + I*Rational(1,2)*3**Rational(1,2),
                             Rational(-1,2) - I*Rational(1,2)*3**Rational(1,2)]
    assert roots(x**3, x) == [0, 0, 0]
    assert roots(x**3-x, x) == [-1, 0, 1]
    assert roots(Rational(2), x) == []
    assert roots(a*x**2 + b*x + c, x) == \
           [-b/(a*2)+((b**2-4*a*c)**Rational(1,2))/2/a,
            -b/(a*2)-((b**2-4*a*c)**Rational(1,2))/2/a]
    assert roots(x**3 + x**2 + x + 1, x) == [-1, I, -I]
    assert roots(x**4 - 1, x) == [-1, 1, I, -I]
    assert roots(x**4 + 1, x) == [((-1)**Rational(1,4)).expand(complex=True),
                               ((-1)**Rational(3,4)).expand(complex=True),
                               (-(-1)**Rational(1,4)).expand(complex=True),
                               (-(-1)**Rational(3,4)).expand(complex=True)]
    assert roots(x**8 - 1, x) == [-1, 1,
        2**Rational(1,2)/2 + I*2**Rational(1,2)/2,
        I,
        -2**Rational(1,2)/2 + I*2**Rational(1,2)/2,
        -2**Rational(1,2)/2 - I*2**Rational(1,2)/2,
        -I,
        2**Rational(1,2)/2 - I*2**Rational(1,2)/2]
    assert roots(x**5 - Rational(3,2), x) == \
           [Rational(1,2)**Rational(1,5)*3**Rational(1,5),
            Rational(1,2)**Rational(1,5)*3**Rational(1,5)*cos(2*pi/5)
            + I*Rational(1,2)**Rational(1,5)*3**Rational(1,5)*sin(2*pi/5),
            Rational(1,2)**Rational(1,5)*3**Rational(1,5)*cos(4*pi/5)
            + I*Rational(1,2)**Rational(1,5)*3**Rational(1,5)*sin(4*pi/5),
            Rational(1,2)**Rational(1,5)*3**Rational(1,5)*cos(6*pi/5)
            + I*Rational(1,2)**Rational(1,5)*3**Rational(1,5)*sin(6*pi/5),
            Rational(1,2)**Rational(1,5)*3**Rational(1,5)*cos(8*pi/5)
            + I*Rational(1,2)**Rational(1,5)*3**Rational(1,5)*sin(8*pi/5)]



import py
from sympy import *
from sympy.numerics.evalf_ import evalf, polyfunc
from sympy.utilities.pytest import XFAIL

@XFAIL
def test_simple_evalf():
    assert evalf(2) == 2
    assert evalf(Rational(1,2)) == 0.5
    assert evalf(exp(pi*I)).ae(-1)
    assert evalf(pi+E).ae(5.8598744820488378)

def test_polyfunc():
    x = Symbol('x')
    p = polyfunc(2*x**3 - 2.5*x + 4)
    q = lambda t: 2*t**3 - 2.5*t + 4
    assert p(0).ae(q(0))
    assert p(1).ae(q(1))
    assert p(-7.5).ae(q(-7.5))
    p = x**3 - 5*x**2 + 4*x - 6
    pd = p.diff(x)
    assert polyfunc(p)(2) == p.subs(x, 2)
    assert polyfunc(p, True)(2) == (p.subs(x, 2), pd.subs(x, 2))



"""
Routines for arbitrary-precision numerical quadrature
"""

from sympy import S, oo

#----------------------------------------------------------------------------#
#                        General classes / utilities                         #
#----------------------------------------------------------------------------#

from float_ import Float
from constants import *
from functions import *
from evalf_ import polyfunc
from sympy import legendre

import math


class Quadrature:
    """
    This class provides a standard interface for quadrature rules. A
    rule is applied through a function call (e.g. to integrate sin(x)
    from x=3 to x=8, SomeQuadRule(<options>)(lambda x: sin(x), 3, 8))

    Each Quadrature subclass should define an '_eval' method that
    returns a tuple containing two values: (y, err) where y is the
    calculated value of the integral, and err is an estimate of
    the approximation error (or None if the error cannot be estimated
    from the rule).

    By default, the quadrature rule is assumed to be defined on
    [-1, 1] or (-1, 1). __call__ automatically transforms other
    (finite) intervals to this default interval. This behavior can
    be modified by overriding the 'transform' method.
    """

    def transform(self, f, a, b):
        """Given an integrand function f and boundaries a, b,
        return an equivalent integrand defined on [-1, 1]"""
        a = Float(a)
        b = Float(b)
        if (a, b) == (-1, 1):
            return f
        else:
            C = (b-a)/2
            D = (b+a)/2
            def g(x):
                return C * f(D + C*x)
            return g

    def __call__(self, f, a, b, extraprec=3, verbose=False):
        Float.store()
        Float.extraprec(extraprec)
        f = self.transform(f, a, b)
        try:
            s, err = self._eval(f, verbose)
        except Exception, e:
            Float.revert()
            raise e
        Float.revert()
        return +s, err

    def adaptive(self, f, a, b, eps, steps=0, maxsteps=5000, verbose=False):
        """Apply rule adaptively (must support error estimation)"""
        s, err = self(f, a, b, verbose=verbose)
        if err <= eps or steps >= maxsteps:
            return s, err, steps+1
        if verbose:
            print steps, a, b
        mid = (a+b)/2
        s1, e1, steps = self.adaptive(f, a, mid, eps, steps+1, maxsteps, verbose)
        s2, e2, steps = self.adaptive(f, mid, b, eps, steps, maxsteps, verbose)
        return s1 + s2, e1 + e2, steps


#----------------------------------------------------------------------------#
#                       Trivial quadrature rules                             #
#----------------------------------------------------------------------------#

# Mainly for demonstration purposes...

class Midpoint(Quadrature):
    def _eval(self, f, verbose=False):
        return 2*f(Float(0)), None

class Trapezoid(Quadrature):
    def _eval(self, f, verbose=False):
        return f(Float(-1)) + f(Float(1)), None


#----------------------------------------------------------------------------#
#                          Gaussian quadrature                               #
#----------------------------------------------------------------------------#

def _iterfixpoint(h, x, eps):
    while 1:
        new = h(x)
        if x.ae(new, eps):
            return new
        x = new

def _gaussnode(pf, k, n):
    import math
    x = Float(-math.cos(math.pi*(k+1-0.25)/(n+0.5)))
    def h(r):
        p, q = pf(r)
        return r - p/q
    eps = Float((1, -Float._prec//2))
    x = _iterfixpoint(h, x, eps)
    w = 2 / (1-x**2) / (pf(x)[1])**2
    return x, w

_gausscache = {}


class GaussLegendre(Quadrature):
    """
    Gauss-Legendre quadrature is highly efficient for smooth integrands
    on finite intervals.
    """

    def __init__(self, n=None, verbose=False):
        self.n = n
        prec = dps = Float.getdps()
        self.prec = prec

        # Reuse old nodes
        if n in _gausscache:
            cacheddps, xs, ws = _gausscache[n]
            if cacheddps >= dps:
                self.x = [x for x in xs]
                self.w = [w for w in ws]
                return

        if verbose:
            print ("calculating nodes for degree-%i Gauss-Legendre "
                "quadrature..." % n)

        Float.store()
        Float.setdps(2*prec + 5)

        self.x = [None] * n
        self.w = [None] * n

        pf = polyfunc(legendre(n, 'x'), True)

        for k in xrange(n//2 + 1):
            if verbose and k % 4 == 0:
                print "  node", k, "of", n//2
            x, w = _gaussnode(pf, k, n)
            self.x[k] = x
            self.x[n-k-1] = -x
            self.w[k] = self.w[n-k-1] = w

        _gausscache[n] = (dps, self.x, self.w)

        Float.revert()

    def _eval(self, f, verbose=False):
        s = Float(0)
        for i in xrange(self.n):
            s += self.w[i] * f(self.x[i])
        return s, None


class CompositeGaussLegendre(Quadrature):
    """
    Gauss-Legendre quadrature with error estimation. Two Gauss-Legendre
    rules of different degree are used, and the error is estimated as
    the difference between the two results.
    """

    def __init__(self, deg1, deg2, verbose=False):
        self.rule1 = GaussLegendre(deg1, verbose)
        self.rule2 = GaussLegendre(deg2, verbose)

    def _eval(self, f, verbose=False):
        s = self.rule1._eval(f, verbose)[0]
        t = self.rule2._eval(f, verbose)[0]
        return s, abs(s-t)


#----------------------------------------------------------------------------#
#                            Tanh-sinh quadrature                            #
#----------------------------------------------------------------------------#

def _tsnode(k, h):
    # x[k] = tanh(pi/2 * sinh(k*h))
    # w[k] = pi/2 * cosh(k*h) / cosh(pi/2 sinh(k*h))**2
    t = Float(k) * h
    a = exp(t)
    pi = pi_float()
    ar = Float(1) / a
    sinht = (a - ar) / 2
    cosht = (a + ar) / 2
    b = exp((pi * sinht) / 2)
    br = Float(1) / b
    sinhb = (b - br) / 2
    coshb = (b + br) / 2
    return sinhb/coshb, (pi/2)*cosht/coshb**2

_tscache = {}

class TanhSinh(Quadrature):
    """
    The tanh-sinh quadrature rule (also known as the doubly-exponential
    quadrature rule) is well suited for integrals with singularities
    and/or extremely high precision levels (tens or hundreds of digits).

    The level m is a small integer between, say, 5 for low precision
    levels and 10 for extremely high precision (it must be set higher
    if the integrand is ill-behaved).

    This implementation of the tanh-sinh algorithm is based on the
    description given in Borwein, Bailey & Girgensohn, "Experimentation
    in Mathematics - Computational Paths to Discovery", A K Peters,
    2003, pages 312-313. It also described in various documents
    available online.
    """

    def __init__(self, eps, m, verbose=False):
        # Compute abscissas and weights

        prec = Float.getdps()

        self.prec = prec
        self.eps = eps
        self.m = m
        self.h = h = Float(1) / 2**m
        self.x = []
        self.w = []

        if (eps, m) in _tscache:
            self.x, self.w = _tscache[(eps, m)]
            return

        if verbose:
            print ("calculating nodes for tanh-sinh quadrature with "
                "epsilon %s and degree %i" % (eps, m))

        Float.store()
        Float.setdps(prec + 10)

        for k in xrange(20 * 2**m + 1):
            x, w = _tsnode(k, h)
            self.x.append(x)
            self.w.append(w)
            diff = abs(self.x[-1] - Float(1))
            if diff <= eps:
                break
            if verbose and m > 6 and k % 300 == 299:
                Float.store(); Float.setdps(5)
                print "  progress", -log(diff, 10) / prec
                Float.revert()

        _tscache[(eps, m)] = self.x, self.w

        Float.revert()

    def _estimate_error(self, res):
        try:
            D1 = log(abs(res[-1]-res[-2]), 10)
            D2 = log(abs(res[-1]-res[-3]), 10)
        except ValueError:
            return self.eps
        D3 = -self.prec
        D4 = min(0, max(D1**2/D2, 2*D1, D3))
        return Float('0.1') ** -int(D4)

    def _eval(self, f, verbose=False):
        S = Float(0)
        h = Float(1)
        m = self.m
        res = []
        for k in xrange(1, m+1):
            if verbose:
                print "tanh-sinh sum level", k, "of", m
            h = h / 2
            for i in xrange(0, len(self.x), 2**(m-k)):
                if i % (2**(m-k+1)) != 0 or k == 1:
                    if i == 0:
                        S = S + self.w[0]*f(Float(0))
                    else:
                        S = S + (self.w[i])*(f(-self.x[i]) + f(self.x[i]))
            res.append(h*S)
            if k > 2:
                err = self._estimate_error(res)
                if err <= self.eps:
                    break
        return +res[-1], self._estimate_error(res)


class AdaptiveTanhSinh(Quadrature):
    """
    Adaptive tanh-sinh quadrature algorithm.
    """

    def __init__(self, initial):
        self.initial = initial

    def adaptive(self, f, a, b, eps, steps=0, maxsteps=5000, verbose=False):
        prec = Float.getprec()
        for m in xrange(self.initial, 50):
            if verbose:
                print "using tanh-sinh rule with m =", m
            ts = TanhSinh(eps, m, verbose=verbose)
            s, err = ts(f, a, b, verbose=verbose)
            steps = 2*len(ts.x)
            if err <= eps or steps >= maxsteps:
                return s, err, steps
            if verbose:
                print "error = ", err


#----------------------------------------------------------------------------#
#                          User-friendly functions                           #
#----------------------------------------------------------------------------#


def nintegrate(f, a, b, method=0, maxsteps=5000, verbose=False):
    """
    Basic usage
    ===========

    nintegrate(f, a, b) numerically evaluates the integral

           - b
          |    f(x) dx.
         -   a


    The integrand f should be a callable that accepts a Float as input
    and outputs a Float or ComplexFloat; a and b should be Floats,
    numbers that can be converted to Floats, or +/- infinity.

    A simple example:

        >>> Float.setdps(15)
        >>> print nintegrate(lambda x: 2*x**2 - 4*x, 2, 3)
        2.66666666666667

    Calculating the area of a unit circle, with 30 digits:

        >>> Float.setdps(30)
        >>> print nintegrate(lambda x: 4*sqrt(1-x**2), 0, 1)
        3.14159265358979323846264338328

    The integration interval can be infinite or semi-infinite:

        >>> Float.setdps(15)
        >>> print nintegrate(lambda x: exp(-x)*sin(x), 0, oo)
        0.5

    Integration methods and accuracy
    ================================

    Nintegrate attempts to obtain a value that is fully accurate within
    the current working precision (i.e., correct to 15 decimal places
    at the default precision level). If nintegrate fails to reach full
    accuracy after a certain number of steps, it prints a warning
    message.

    This message signifies either that the integral is either divergent
    or, if convergent, ill-behaved. It may still be possible to
    evaluate an ill-behaved integral by increasing the 'maxsteps'
    setting, changing the integration method, and/or manually
    transforming the integrand.

    Nintegrate currently supports the following integration methods:

        method = 0  :  Gaussian quadrature (default)
        method = 1  :  tanh-sinh quadrature

    Gaussian quadrature is generally very efficient if the integration
    interval is finite and the integrand is smooth on the entire range
    (including the endpoints). It may fail if the integrand has many
    discontinuities, is highly oscillatory, or possesses integrable
    singularities.

    The tanh-sinh algorithm is often better if the integration interval
    is infinite or if singularities are present at the endpoints;
    especially at very high precision levels. It does not perform well
    if there are singularities between the endpoints or the integrand
    is bumpy or oscillatory.

    It may help to manually transform the integrand, e.g. changing
    variables to remove singularities or breaking up the integration
    interval so that singularities appear only at the endpoints.

    The 'verbose' flag can be set to track the computation's progress.
    """
    dps = Float.getdps()
    Float.store()
    Float.setdps(dps + 3)
    prec = Float.getprec()

    # Transform infinite or semi-infinite interval to a finite interval
    if a == -oo or b == oo:
        g = f
        if a == -oo and b == oo:
            def f(x):
                # make adaptive quadrature work from the left
                x = 1 - x
                return g(x) + g(Float(1)/x)/x**2 + g(-x) + g(Float(-1)/x)/(-x)**2
        elif b == oo:
            aa = Float(a)
            def f(x):
                x = 1 - x
                return g(x + aa) + g(Float(1)/x + aa)/x**2
        elif a == -oo:
            bb = Float(b)
            def f(x):
                x = 1 - x
                return g(-x + bb) + g(Float(-1)/x + bb)/(-x)**2
        a, b = Float(0), Float(1)
    else:
        a, b = Float(a), Float(b)

    eps = Float((1, -prec+4))

    if method == 0:
        degree = int(5 + dps**0.8)
        rule = CompositeGaussLegendre(degree, degree//2, verbose)
    elif method == 1:
        rule = AdaptiveTanhSinh(initial = int(3 + max(0, math.log(prec/30.0, 2))))

    else:
        Float.revert()
        raise ValueError("unknown method")

    s, err, steps = rule.adaptive(f, Float(a), Float(b), eps, steps=0, maxsteps=maxsteps, verbose=verbose)

    Float.revert()

    if not err.ae(0):
        Float.store()
        Float.setdps(1)
        print "Warning: failed to reach full accuracy.", \
            "Estimated magnitude of error: ", str(err)
        Float.revert()

    return +s



# ----------------------------------------------------------------------------
# pyglet
# Copyright (c) 2006-2007 Alex Holkner
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met:
#
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  * Neither the name of the pyglet nor the names of its
#    contributors may be used to endorse or promote products
#    derived from this software without specific prior written
#    permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------

'''Cached information about version and extensions of current WGL
implementation.
'''

__docformat__ = 'restructuredtext'
__version__ = '$Id: glx_info.py 615 2007-02-07 13:17:05Z Alex.Holkner $'

from ctypes import *
import warnings

from pyglet.gl.lib import MissingFunctionException
from pyglet.gl.gl import *
from pyglet.gl import gl_info
from pyglet.gl.wgl import *
from pyglet.gl.wglext_arb import *

class WGLInfoException(Exception):
    pass

class WGLInfo(object):
    def get_extensions(self):
        if not gl_info.have_context():
            warnings.warn("Can't query WGL until a context is created.")
            return []

        try:
            return wglGetExtensionsStringEXT().split()
        except MissingFunctionException:
            return cast(glGetString(GL_EXTENSIONS), c_char_p).value.split()

    def have_extension(self, extension):
        return extension in self.get_extensions()

_wgl_info = WGLInfo()

get_extensions = _wgl_info.get_extensions
have_extension = _wgl_info.have_extension



from sympy import Symbol, exp, Integer, Real, sin, cos, log, Polynomial, Lambda, Function, I
from sympy.abc import x, y
from sympy.core.sympify import sympify, _sympify, _sympifyit, SympifyError
import py
#from sympy.utilities.pytest import XFAIL


def test_439():
    v = sympify("exp(x)")
    x = Symbol("x")
    assert v == exp(x)
    assert type(v) == type(exp(x))
    assert str(type(v)) == str(type(exp(x)))

def test_sympify1():
    assert sympify("x") == Symbol("x")
    assert sympify("   x") == Symbol("x")
    assert sympify("   x   ") == Symbol("x")

def test_sympify2():
    class A:
        def _sympy_(self):
            return Symbol("x")**3

    a = A()

    assert _sympify(a)== x**3
    assert sympify(a) == x**3
    assert a == x**3

def test_sympify3():
    assert sympify("x**3") == x**3
    assert sympify("1/2") == Integer(1)/2

    py.test.raises(SympifyError, "_sympify('x**3')")
    py.test.raises(SympifyError, "_sympify('1/2')")

def test_sympify4():
    class A:
        def _sympy_(self):
            return Symbol("x")

    a = A()

    assert _sympify(a)**3== x**3
    assert sympify(a)**3 == x**3
    assert a == x


def test_sympify_poly():
    p = Polynomial(1+x+x**2)

    assert _sympify(p) is p
    assert sympify(p) is p

def test_sage():
    # how to effectivelly test for the _sage_() method without having SAGE
    # installed?
    assert hasattr(x, "_sage_")
    assert hasattr(Integer(3), "_sage_")
    assert hasattr(sin(x), "_sage_")
    assert hasattr(cos(x), "_sage_")
    assert hasattr(x**2, "_sage_")
    assert hasattr(x+y, "_sage_")
    assert hasattr(exp(x), "_sage_")
    assert hasattr(log(x), "_sage_")

def test_bug496():
    a_ = sympify("a_")
    _a = sympify("_a")

def test_lambda():
    x = Symbol('x')
    assert sympify('lambda : 1')==Lambda(x, 1)
    assert sympify('lambda x: 2*x')==Lambda(x, 2*x)

    py.test.raises(SympifyError, "_sympify('lambda : 1')")

def test_sympify_raises():
    py.test.raises(SympifyError, sympify, "fx)")


def test__sympify():
    x = Symbol('x')
    f = Function('f')

    # positive _sympify
    assert _sympify(x)      is x
    assert _sympify(f)      is f
    assert _sympify(1)      == Integer(1)
    assert _sympify(0.5)    == Real("0.5")
    assert _sympify(1+1j)   == 1 + I

    class A:
        def _sympy_(self):
            return Integer(5)

    a = A()
    assert _sympify(a)      == Integer(5)

    # negative _sympify
    py.test.raises(SympifyError, "_sympify('1')")
    py.test.raises(SympifyError, "_sympify([1,2,3])")


def test_sympifyit():
    x = Symbol('x')
    y = Symbol('y')

    @_sympifyit('b', NotImplemented)
    def add(a, b):
        return a+b

    assert add(x, 1)    == x+1
    assert add(x, 0.5)  == x+Real('0.5')
    assert add(x, y)    == x+y

    assert add(x, '1')  == NotImplemented


    @_sympifyit('b')
    def add_raises(a, b):
        return a+b

    assert add_raises(x, 1)     == x+1
    assert add_raises(x, 0.5)   == x+Real('0.5')
    assert add_raises(x, y)     == x+y

    py.test.raises(SympifyError, "add_raises(x, '1')")



import py
#from sympy import *
from sympy.numerics import *
from sympy.numerics.functions import *
import math
import cmath
from sympy.utilities.pytest import XFAIL

def test_sqrt():
    for i in range(1000):
        assert sqrt(Float(i**2)) == i
    # These should round identically
    for x in [0, 1e-7, 0.1, 0.5, 1, 2, 3, 4, 5, 0.333, 76.19]:
        assert sqrt(Float(x)) == float(x)**0.5
    assert sqrt(-1) == 1j
    assert sqrt(-2).ae(cmath.sqrt(-2))
    assert sqrt(-3).ae(cmath.sqrt(-3))
    assert sqrt(-100).ae(cmath.sqrt(-100))
    assert sqrt(1j).ae(cmath.sqrt(1j))
    assert sqrt(-1j).ae(cmath.sqrt(-1j))
    assert sqrt(math.pi + math.e*1j).ae(cmath.sqrt(math.pi + math.e*1j))
    assert sqrt(math.pi - math.e*1j).ae(cmath.sqrt(math.pi - math.e*1j))

def test_hypot():
    assert hypot(0, 0) == 0
    assert hypot(0, 0.33) == Float(0.33)
    assert hypot(0.33, 0) == Float(0.33)
    assert hypot(-0.33, 0) == Float(0.33)
    assert hypot(3, 4) == Float(5)

def test_exp():
    assert exp(0) == 1
    assert exp(10000).ae(Float('8.8068182256629215873e4342'))
    assert exp(-10000).ae(Float('1.1354838653147360985e-4343'))
    assert exp(log2_float() * Float(10)).ae(1024)
    assert exp(2+2j).ae(cmath.exp(2+2j))

def test_log():
    assert log(1) == 0
    for x in [0.5, 1.5, 2.0, 3.0, 100, 10**50, 1e-50]:
        assert log(x) == math.log(x)
        assert log(x, x) == 1
    assert log(1024, 2) == 10
    assert log(10**1234, 10) == 1234
    assert log(2+2j).ae(cmath.log(2+2j))

@XFAIL
def test_trig_basic():
    for x in (range(100) + range(-100,0)):
        t = x / 4.1
        assert cos(Float(t)).ae(math.cos(t))
        assert sin(Float(t)).ae(math.sin(t))
        assert tan(Float(t)).ae(math.tan(t))
    assert sin(1+1j).ae(cmath.sin(1+1j))
    assert sin(-4-3.6j).ae(cmath.sin(-4-3.6j))

def test_trig_hard():
    assert sin(Float(10**50, 150)).ae(-0.7896724934293100827)
    assert cos(Float(10**50, 150)).ae(-0.6135286082336635622)
    assert sin(1e-6).ae(9.999999999998333e-007)
    assert cos(1e-6).ae(0.9999999999995)

def test_atan():
    import math
    assert atan(-2.3).ae(math.atan(-2.3))
    assert atan2(1,1).ae(math.atan2(1,1))
    assert atan2(1,-1).ae(math.atan2(1,-1))
    assert atan2(-1,-1).ae(math.atan2(-1,-1))
    assert atan2(-1,1).ae(math.atan2(-1,1))
    assert atan2(-1,0).ae(math.atan2(-1,0))
    assert atan2(1,0).ae(math.atan2(1,0))



from sympy import Symbol, floor, nan, oo, E, symbols, ceiling, pi, Rational, \
        Real, I, sin

def test_floor():

    x = Symbol('x')
    y = Symbol('y', real=True)
    k, n = symbols('kn', integer=True)

    assert floor(nan) == nan

    assert floor(oo) == oo
    assert floor(-oo) == -oo

    assert floor(0) == 0

    assert floor(1) == 1
    assert floor(-1) == -1

    assert floor(E) == 2
    assert floor(-E) == -3

    assert floor(2*E) == 5
    assert floor(-2*E) == -6

    assert floor(pi) == 3
    assert floor(-pi) == -4

    assert floor(Rational(1, 2)) == 0
    assert floor(-Rational(1, 2)) == -1

    assert floor(Rational(7, 3)) == 2
    assert floor(-Rational(7, 3)) == -3

    assert floor(Real(17.0)) == 17
    assert floor(-Real(17.0)) == -17

    assert floor(Real(7.69)) == 7
    assert floor(-Real(7.69)) == -8

    assert floor(I) == I
    assert floor(-I) == -I

    assert floor(oo*I) == oo*I
    assert floor(-oo*I) == -oo*I

    assert floor(2*I) == 2*I
    assert floor(-2*I) == -2*I

    assert floor(I/2) == 0
    assert floor(-I/2) == -I

    assert floor(E + 17) == 19
    assert floor(pi + 2) == 5

    assert floor(E + pi) == floor(E + pi)
    assert floor(I + pi) == floor(I + pi)

    assert floor(floor(pi)) == 3
    assert floor(floor(y)) == floor(y)
    assert floor(floor(x)) == floor(floor(x))

    assert floor(x) == floor(x)
    assert floor(2*x) == floor(2*x)
    assert floor(k*x) == floor(k*x)

    assert floor(k) == k
    assert floor(2*k) == 2*k
    assert floor(k*n) == k*n

    assert floor(k/2) == floor(k/2)

    assert floor(x + y) == floor(x + y)

    assert floor(x + 3) == floor(x + 3)
    assert floor(x + k) == floor(x + k)

    assert floor(y + 3) == floor(y) + 3
    assert floor(y + k) == floor(y) + k

    assert floor(k + n) == k + n

    assert floor(x*I) == floor(x*I)
    assert floor(k*I) == k*I

    assert floor(Rational(23, 10) - E*I) == 2 - 3*I

    assert floor(sin(1)) == 0
    assert floor(sin(-1)) == -1

def test_ceiling():

    x = Symbol('x')
    y = Symbol('y', real=True)
    k, n = symbols('kn', integer=True)

    assert ceiling(nan) == nan

    assert ceiling(oo) == oo
    assert ceiling(-oo) == -oo

    assert ceiling(0) == 0

    assert ceiling(1) == 1
    assert ceiling(-1) == -1

    assert ceiling(E) == 3
    assert ceiling(-E) == -2

    assert ceiling(2*E) == 6
    assert ceiling(-2*E) == -5

    assert ceiling(pi) == 4
    assert ceiling(-pi) == -3

    assert ceiling(Rational(1, 2)) == 1
    assert ceiling(-Rational(1, 2)) == 0

    assert ceiling(Rational(7, 3)) == 3
    assert ceiling(-Rational(7, 3)) == -2

    assert ceiling(Real(17.0)) == 17
    assert ceiling(-Real(17.0)) == -17

    assert ceiling(Real(7.69)) == 8
    assert ceiling(-Real(7.69)) == -7

    assert ceiling(I) == I
    assert ceiling(-I) == -I

    assert ceiling(oo*I) == oo*I
    assert ceiling(-oo*I) == -oo*I

    assert ceiling(2*I) == 2*I
    assert ceiling(-2*I) == -2*I

    assert ceiling(I/2) == I
    assert ceiling(-I/2) == 0

    assert ceiling(E + 17) == 20
    assert ceiling(pi + 2) == 6

    assert ceiling(E + pi) == ceiling(E + pi)
    assert ceiling(I + pi) == ceiling(I + pi)

    assert ceiling(ceiling(pi)) == 4
    assert ceiling(ceiling(y)) == ceiling(y)
    assert ceiling(ceiling(x)) == ceiling(ceiling(x))

    assert ceiling(x) == ceiling(x)
    assert ceiling(2*x) == ceiling(2*x)
    assert ceiling(k*x) == ceiling(k*x)

    assert ceiling(k) == k
    assert ceiling(2*k) == 2*k
    assert ceiling(k*n) == k*n

    assert ceiling(k/2) == ceiling(k/2)

    assert ceiling(x + y) == ceiling(x + y)

    assert ceiling(x + 3) == ceiling(x + 3)
    assert ceiling(x + k) == ceiling(x + k)

    assert ceiling(y + 3) == ceiling(y) + 3
    assert ceiling(y + k) == ceiling(y) + k

    assert ceiling(k + n) == k + n

    assert ceiling(x*I) == ceiling(x*I)
    assert ceiling(k*I) == k*I

    assert ceiling(Rational(23, 10) - E*I) == 3 - 2*I

    assert ceiling(sin(1)) == 1
    assert ceiling(sin(-1)) == 0




def npartitions(n):
    """
    Calculate the partition function P(n), i.e. the number of ways that
    n can be written as a sum of positive integers.

    P(n) is computed using a straightforward implementation of the
    Hardy-Ramanujan-Rademacher formula, described e.g. at
    http://mathworld.wolfram.com/PartitionFunctionP.html

    The speed is decent up to n = 10**5 or so. The function has
    been tested to give the correct result for n = 10**6.
    """

    n = int(n)
    if n < 0:
        return 0
    if n <= 5:
        return [1, 1, 2, 3, 5, 7][n]

    from sympy.core.numbers import gcd
    from sympy.numerics import Float
    from sympy.numerics.functions import pi_float, sqrt, exp, log, cos

    def frac(x):
        return x - int(x)

    def D(n, j):
        pi = pi_float()
        a = sqrt(Float(2)/3) * pi / j
        b = Float(n) - Float(1)/24
        c = sqrt(b)
        expa = exp(a*c)
        iexpa = Float(1)/expa
        ch = (expa + iexpa)*0.5
        sh = (expa - iexpa)*0.5
        return sqrt(j) / (2*sqrt(2)*b*pi) * (a*ch-sh/c)

    def A(n, j):
        if j == 1:
            return Float(1)
        s = Float(0)
        pi = pi_float()
        for h in xrange(1, j):
            if gcd(h,j) == 1:
                s += cos((g(h,j)-2*h*n)*pi/j)
        return s

    def g(h, j):
        if j < 3:
            return Float(0)
        s = Float(0)
        for k in xrange(1, j):
            s += k*(frac(h*Float(k)/j)-0.5)
        return s

    # estimate number of digits in p(n)
    pdigits = int((pi_float()*sqrt(2.0*n/3)-log(4*n))/log(10)+1)
    Float.store()
    Float.setdps(pdigits*1.1 + 10)
    s = Float(0)

    M = max(6, int(0.24*sqrt(n)+4))
    for q in xrange(1, M):
        s += A(n,q) * D(n,q)
    p = int(s + 0.5)
    Float.revert()

    return p



from pyglet.gl import *
from plot_mode_base import PlotModeBase
from sympy.core.basic import S
from util import scale_value, scale_value_list
#from time import sleep

class PlotSurface(PlotModeBase):

    default_rot_preset = 'perspective'

    def _on_calculate_verts(self):
        self.u_interval = self.intervals[0]
        self.u_set = list(self.u_interval.frange())
        self.v_interval = self.intervals[1]
        self.v_set = list(self.v_interval.frange())
        self.bounds = [ [S.Infinity,-S.Infinity,0],[S.Infinity,-S.Infinity,0],[S.Infinity,-S.Infinity,0] ]
        evaluate = self._get_evaluator()

        self._calculating_verts_pos = 0.0
        self._calculating_verts_len = float(self.u_interval.v_len*self.v_interval.v_len)

        verts = list()
        b = self.bounds
        for u in self.u_set:
            column = list()
            for v in self.v_set:
                try: _e = evaluate(u, v) # calculate vertex
                except: _e = None
                if _e is not None:  # update bounding box
                    for axis in xrange(3):
                        b[axis][0] = min([b[axis][0], _e[axis]])
                        b[axis][1] = max([b[axis][1], _e[axis]])
                column.append(_e)
                self._calculating_verts_pos += 1.0

            verts.append(column)
        for axis in xrange(3):
            b[axis][2] = b[axis][1] - b[axis][0]
            if b[axis][2] == 0.0: b[axis][2] = 1.0

        self.verts = verts
        self.push_wireframe(self.draw_verts(False, False))
        self.push_solid(self.draw_verts(False, True))

    def _on_calculate_cverts(self):
        if not self.verts or not self.color: return
        def set_work_len(n): self._calculating_cverts_len = float(n)
        def inc_work_pos(): self._calculating_cverts_pos += 1.0
        set_work_len(1); self._calculating_cverts_pos = 0
        self.cverts = self.color.apply_to_surface(self.verts, self.u_set, self.v_set,
                                                  set_len=set_work_len, inc_pos=inc_work_pos)
        self.push_solid(self.draw_verts(True, True))

    def calculate_one_cvert(self, u, v):
        vert = self.verts[u][v]
        return self.color(vert[0], vert[1], vert[2],
                          self.u_set[u], self.v_set[v])

    def draw_verts(self, use_cverts, use_solid_color):
        def f():
            for u in xrange( 1, len(self.u_set) ):
                glBegin(GL_QUAD_STRIP)
                for v in xrange( len(self.v_set) ):
                    pa = self.verts[u-1][v]
                    pb = self.verts[u][v]
                    if pa is None or pb is None:
                        glEnd()
                        glBegin(GL_QUAD_STRIP)
                        continue
                    if use_cverts:
                        ca = self.cverts[u-1][v]
                        cb = self.cverts[u][v]
                        if ca is None:
                            ca = (0,0,0)
                        if cb is None:
                            cb = (0,0,0)
                    else:
                        if use_solid_color: ca = cb = self.default_solid_color
                        else: ca = cb = self.default_wireframe_color
                    glColor3f(*ca)
                    glVertex3f(*pa)
                    glColor3f(*cb)
                    glVertex3f(*pb)
                glEnd()
        return f




"""
This module contains functions and classes for arbitrary-precision
numerical computations.

>>> from sympy.numerics import Float
>>> from sympy.numerics.functions import exp, log, sqrt
>>> from sympy.numerics.constants import pi_float
>>> Float.setdps(50)
>>> print pi_float()
3.1415926535897932384626433832795028841971693993751
>>> print exp(1)
2.7182818284590452353602874713526624977572470937000
>>> print log(2)
0.69314718055994530941723212145817656807550013436025
>>> print sqrt(2)
1.4142135623730950488016887242096980785696718753769

"""

from float_ import *
#from constants import *
#from functions import *
#from functions2 import *
from optimize import polyroots, bisect, secant
from quad import nintegrate
from evalf_ import evalf, polyfunc
